@@ -43,28 +43,6 @@
 # include <Zend/zend_string.h>
 #endif
 
-#if PHP_VERSION_ID >= 50500
-#define ITERATOR_GET_CURRENT_KEY(it, z) \
-		(it).funcs->get_current_key(&(it), z TSRMLS_CC);
-#else
-#define ITERATOR_GET_CURRENT_KEY(it, z) \
-{ \
-	char *str_key; \
-	uint str_key_len; \
-	ulong int_key; \
-	switch ((it).funcs->get_current_key(&(it), &str_key, &str_key_len, &int_key TSRMLS_CC)) { \
-		case HASH_KEY_IS_LONG: \
-			ZVAL_LONG(z, int_key); \
-			break; \
-		case HASH_KEY_IS_STRING: \
-			ZVAL_STRINGL(z, str_key, str_key_len-1, 0); \
-			break; \
-		default: \
-			RETURN_NULL(); \
-	} \
-}
-#endif
-
 #if defined(__GNUC__)
 #  define ARG_UNUSED  __attribute__ ((unused))
 #else
@@ -83,6 +61,7 @@
 #endif
 # define SIZEOF_PHONGO_LONG SIZEOF_ZEND_LONG
 # define phongo_create_object_retval zend_object*
+# define phongo_get_gc_table zval **
 # define PHONGO_ALLOC_OBJECT_T(_obj_t, _class_type) (_obj_t *)ecalloc(1, sizeof(_obj_t)+zend_object_properties_size(_class_type))
 # define PHONGO_TSRMLS_FETCH_FROM_CTX(user_data)
 # define SUPPRESS_UNUSED_WARNING(x)
@@ -112,6 +91,7 @@
 # define SIZEOF_PHONGO_LONG SIZEOF_LONG
 # define ZSTR_VAL(str) str
 # define phongo_create_object_retval zend_object_value
+# define phongo_get_gc_table zval ***
 # define PHONGO_ALLOC_OBJECT_T(_obj_t, _class_type) (_obj_t *)ecalloc(1, sizeof(_obj_t))
 # define PHONGO_TSRMLS_FETCH_FROM_CTX(user_data) TSRMLS_FETCH_FROM_CTX(user_data)
 # define SUPPRESS_UNUSED_WARNING(x) (void)x;
@@ -128,6 +108,7 @@
 # define ADD_NEXT_INDEX_STRINGL(_zv, _value, _len) add_next_index_stringl(_zv, _value, _len, 1);
 # define Z_PHPDATE_P(object) ((php_date_obj*)zend_object_store_get_object(object TSRMLS_CC))
 # define Z_ISUNDEF(x) !x
+# define ZVAL_UNDEF(x) do { (*x) = NULL; } while (0)
 # define phongo_free_object_arg void
 # define phongo_zpp_char_len int
 # define ZEND_HASH_APPLY_PROTECTION(ht) true
