@@ -91,6 +91,15 @@ static inline php_phongo_timestamp_t* php_timestamp_fetch_object(zend_object *ob
 static inline php_phongo_utcdatetime_t* php_utcdatetime_fetch_object(zend_object *obj) {
     return (php_phongo_utcdatetime_t *)((char *)obj - XtOffsetOf(php_phongo_utcdatetime_t, std));
 }
+static inline php_phongo_commandfailedevent_t* php_commandfailedevent_fetch_object(zend_object *obj) {
+    return (php_phongo_commandfailedevent_t *)((char *)obj - XtOffsetOf(php_phongo_commandfailedevent_t, std));
+}
+static inline php_phongo_commandstartedevent_t* php_commandstartedevent_fetch_object(zend_object *obj) {
+    return (php_phongo_commandstartedevent_t *)((char *)obj - XtOffsetOf(php_phongo_commandstartedevent_t, std));
+}
+static inline php_phongo_commandsucceededevent_t* php_commandsucceededevent_fetch_object(zend_object *obj) {
+    return (php_phongo_commandsucceededevent_t *)((char *)obj - XtOffsetOf(php_phongo_commandsucceededevent_t, std));
+}
 
 # define Z_COMMAND_OBJ_P(zv)           (php_command_fetch_object(Z_OBJ_P(zv)))
 # define Z_CURSOR_OBJ_P(zv)            (php_cursor_fetch_object(Z_OBJ_P(zv)))
@@ -114,6 +123,9 @@ static inline php_phongo_utcdatetime_t* php_utcdatetime_fetch_object(zend_object
 # define Z_REGEX_OBJ_P(zv)             (php_regex_fetch_object(Z_OBJ_P(zv)))
 # define Z_TIMESTAMP_OBJ_P(zv)         (php_timestamp_fetch_object(Z_OBJ_P(zv)))
 # define Z_UTCDATETIME_OBJ_P(zv)       (php_utcdatetime_fetch_object(Z_OBJ_P(zv)))
+# define Z_COMMANDFAILEDEVENT_OBJ_P(zv)    (php_commandfailedevent_fetch_object(Z_OBJ_P(zv)))
+# define Z_COMMANDSTARTEDEVENT_OBJ_P(zv)   (php_commandstartedevent_fetch_object(Z_OBJ_P(zv)))
+# define Z_COMMANDSUCCEEDEDEVENT_OBJ_P(zv) (php_commandsucceededevent_fetch_object(Z_OBJ_P(zv)))
 
 # define Z_OBJ_COMMAND(zo)             (php_command_fetch_object(zo))
 # define Z_OBJ_CURSOR(zo)              (php_cursor_fetch_object(zo))
@@ -137,6 +149,9 @@ static inline php_phongo_utcdatetime_t* php_utcdatetime_fetch_object(zend_object
 # define Z_OBJ_REGEX(zo)               (php_regex_fetch_object(zo))
 # define Z_OBJ_TIMESTAMP(zo)           (php_timestamp_fetch_object(zo))
 # define Z_OBJ_UTCDATETIME(zo)         (php_utcdatetime_fetch_object(zo))
+# define Z_OBJ_COMMANDFAILEDEVENT(zo)    (php_commandfailedevent_fetch_object(zo))
+# define Z_OBJ_COMMANDSTARTEDEVENT(zo)   (php_commandstartedevent_fetch_object(zo))
+# define Z_OBJ_COMMANDSUCCEEDEDEVENT(zo) (php_commandsucceededevent_fetch_object(zo))
 
 #else
 
@@ -162,6 +177,9 @@ static inline php_phongo_utcdatetime_t* php_utcdatetime_fetch_object(zend_object
 # define Z_REGEX_OBJ_P(zv)             ((php_phongo_regex_t *)zend_object_store_get_object(zv TSRMLS_CC))
 # define Z_TIMESTAMP_OBJ_P(zv)         ((php_phongo_timestamp_t *)zend_object_store_get_object(zv TSRMLS_CC))
 # define Z_UTCDATETIME_OBJ_P(zv)       ((php_phongo_utcdatetime_t *)zend_object_store_get_object(zv TSRMLS_CC))
+# define Z_COMMANDFAILEDEVENT_OBJ_P(zv)    ((php_phongo_commandfailedevent_t *)zend_object_store_get_object(zv TSRMLS_CC))
+# define Z_COMMANDSTARTEDEVENT_OBJ_P(zv)   ((php_phongo_commandstartedevent_t *)zend_object_store_get_object(zv TSRMLS_CC))
+# define Z_COMMANDSUCCEEDEDEVENT_OBJ_P(zv) ((php_phongo_commandsucceededevent_t *)zend_object_store_get_object(zv TSRMLS_CC))
 
 # define Z_OBJ_COMMAND(zo)             ((php_phongo_command_t *)zo)
 # define Z_OBJ_CURSOR(zo)              ((php_phongo_cursor_t *)zo)
@@ -185,6 +203,9 @@ static inline php_phongo_utcdatetime_t* php_utcdatetime_fetch_object(zend_object
 # define Z_OBJ_REGEX(zo)               ((php_phongo_regex_t *)zo)
 # define Z_OBJ_TIMESTAMP(zo)           ((php_phongo_timestamp_t *)zo)
 # define Z_OBJ_UTCDATETIME(zo)         ((php_phongo_utcdatetime_t *)zo)
+# define Z_OBJ_COMMANDFAILEDEVENT(zo)    ((php_phongo_commandfailedevent_t *)zo)
+# define Z_OBJ_COMMANDSTARTEDEVENT(zo)   ((php_phongo_commandstartedevent_t *)zo)
+# define Z_OBJ_COMMANDSUCCEEDEDEVENT(zo) ((php_phongo_commandsucceededevent_t *)zo)
 
 #endif
 
@@ -194,94 +215,123 @@ typedef struct {
 } php_phongo_cursor_iterator;
 
 
-extern PHONGO_API zend_class_entry *php_phongo_command_ce;
-extern PHONGO_API zend_class_entry *php_phongo_cursor_ce;
-extern PHONGO_API zend_class_entry *php_phongo_cursorid_ce;
-extern PHONGO_API zend_class_entry *php_phongo_manager_ce;
-extern PHONGO_API zend_class_entry *php_phongo_query_ce;
-extern PHONGO_API zend_class_entry *php_phongo_readconcern_ce;
-extern PHONGO_API zend_class_entry *php_phongo_readpreference_ce;
-extern PHONGO_API zend_class_entry *php_phongo_result_ce;
-extern PHONGO_API zend_class_entry *php_phongo_server_ce;
-extern PHONGO_API zend_class_entry *php_phongo_bulkwrite_ce;
-extern PHONGO_API zend_class_entry *php_phongo_writeconcern_ce;
-extern PHONGO_API zend_class_entry *php_phongo_writeconcernerror_ce;
-extern PHONGO_API zend_class_entry *php_phongo_writeerror_ce;
-extern PHONGO_API zend_class_entry *php_phongo_writeresult_ce;
+extern zend_class_entry *php_phongo_command_ce;
+extern zend_class_entry *php_phongo_cursor_ce;
+extern zend_class_entry *php_phongo_cursorid_ce;
+extern zend_class_entry *php_phongo_manager_ce;
+extern zend_class_entry *php_phongo_query_ce;
+extern zend_class_entry *php_phongo_readconcern_ce;
+extern zend_class_entry *php_phongo_readpreference_ce;
+extern zend_class_entry *php_phongo_server_ce;
+extern zend_class_entry *php_phongo_bulkwrite_ce;
+extern zend_class_entry *php_phongo_writeconcern_ce;
+extern zend_class_entry *php_phongo_writeconcernerror_ce;
+extern zend_class_entry *php_phongo_writeerror_ce;
+extern zend_class_entry *php_phongo_writeresult_ce;
 
-extern PHONGO_API zend_class_entry *php_phongo_exception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_logicexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_runtimeexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_unexpectedvalueexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_invalidargumentexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_connectionexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_authenticationexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_sslconnectionexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_executiontimeoutexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_connectiontimeoutexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_writeexception_ce;
-extern PHONGO_API zend_class_entry *php_phongo_bulkwriteexception_ce;
+extern zend_class_entry *php_phongo_exception_ce;
+extern zend_class_entry *php_phongo_logicexception_ce;
+extern zend_class_entry *php_phongo_runtimeexception_ce;
+extern zend_class_entry *php_phongo_unexpectedvalueexception_ce;
+extern zend_class_entry *php_phongo_invalidargumentexception_ce;
+extern zend_class_entry *php_phongo_connectionexception_ce;
+extern zend_class_entry *php_phongo_authenticationexception_ce;
+extern zend_class_entry *php_phongo_sslconnectionexception_ce;
+extern zend_class_entry *php_phongo_executiontimeoutexception_ce;
+extern zend_class_entry *php_phongo_connectiontimeoutexception_ce;
+extern zend_class_entry *php_phongo_writeexception_ce;
+extern zend_class_entry *php_phongo_bulkwriteexception_ce;
 
-extern PHONGO_API zend_class_entry *php_phongo_type_ce;
-extern PHONGO_API zend_class_entry *php_phongo_persistable_ce;
-extern PHONGO_API zend_class_entry *php_phongo_unserializable_ce;
-extern PHONGO_API zend_class_entry *php_phongo_serializable_ce;
-extern PHONGO_API zend_class_entry *php_phongo_binary_ce;
-extern PHONGO_API zend_class_entry *php_phongo_decimal128_ce;
-extern PHONGO_API zend_class_entry *php_phongo_javascript_ce;
-extern PHONGO_API zend_class_entry *php_phongo_maxkey_ce;
-extern PHONGO_API zend_class_entry *php_phongo_minkey_ce;
-extern PHONGO_API zend_class_entry *php_phongo_objectid_ce;
-extern PHONGO_API zend_class_entry *php_phongo_regex_ce;
-extern PHONGO_API zend_class_entry *php_phongo_timestamp_ce;
-extern PHONGO_API zend_class_entry *php_phongo_utcdatetime_ce;
+extern zend_class_entry *php_phongo_type_ce;
+extern zend_class_entry *php_phongo_persistable_ce;
+extern zend_class_entry *php_phongo_unserializable_ce;
+extern zend_class_entry *php_phongo_serializable_ce;
+extern zend_class_entry *php_phongo_binary_ce;
+extern zend_class_entry *php_phongo_decimal128_ce;
+extern zend_class_entry *php_phongo_javascript_ce;
+extern zend_class_entry *php_phongo_maxkey_ce;
+extern zend_class_entry *php_phongo_minkey_ce;
+extern zend_class_entry *php_phongo_objectid_ce;
+extern zend_class_entry *php_phongo_regex_ce;
+extern zend_class_entry *php_phongo_timestamp_ce;
+extern zend_class_entry *php_phongo_utcdatetime_ce;
 
-/* Shared across all MongoDB\Driver objects to disable unserialize() */
-PHP_METHOD(Manager, __wakeup);
-/* Shared across all final MongoDB\Driver value objects, only possible to construct them internally */
-PHP_METHOD(Server, __construct);
+extern zend_class_entry *php_phongo_binary_interface_ce;
+extern zend_class_entry *php_phongo_decimal128_interface_ce;
+extern zend_class_entry *php_phongo_javascript_interface_ce;
+extern zend_class_entry *php_phongo_maxkey_interface_ce;
+extern zend_class_entry *php_phongo_minkey_interface_ce;
+extern zend_class_entry *php_phongo_objectid_interface_ce;
+extern zend_class_entry *php_phongo_regex_interface_ce;
+extern zend_class_entry *php_phongo_timestamp_interface_ce;
+extern zend_class_entry *php_phongo_utcdatetime_interface_ce;
 
-PHP_MINIT_FUNCTION(Command);
-PHP_MINIT_FUNCTION(Cursor);
-PHP_MINIT_FUNCTION(CursorId);
-PHP_MINIT_FUNCTION(Manager);
-PHP_MINIT_FUNCTION(Query);
-PHP_MINIT_FUNCTION(ReadConcern);
-PHP_MINIT_FUNCTION(ReadPreference);
-PHP_MINIT_FUNCTION(Result);
-PHP_MINIT_FUNCTION(Server);
-PHP_MINIT_FUNCTION(BulkWrite);
-PHP_MINIT_FUNCTION(WriteConcern);
-PHP_MINIT_FUNCTION(WriteConcernError);
-PHP_MINIT_FUNCTION(WriteError);
-PHP_MINIT_FUNCTION(WriteResult);
+extern zend_class_entry *php_phongo_commandfailedevent_ce;
+extern zend_class_entry *php_phongo_commandstartedevent_ce;
+extern zend_class_entry *php_phongo_commandsubscriber_ce;
+extern zend_class_entry *php_phongo_commandsucceededevent_ce;
+extern zend_class_entry *php_phongo_subscriber_ce;
 
-PHP_MINIT_FUNCTION(Exception);
-PHP_MINIT_FUNCTION(LogicException);
-PHP_MINIT_FUNCTION(RuntimeException);
-PHP_MINIT_FUNCTION(UnexpectedValueException);
-PHP_MINIT_FUNCTION(InvalidArgumentException);
-PHP_MINIT_FUNCTION(ConnectionException);
-PHP_MINIT_FUNCTION(AuthenticationException);
-PHP_MINIT_FUNCTION(SSLConnectionException);
-PHP_MINIT_FUNCTION(ExecutionTimeoutException);
-PHP_MINIT_FUNCTION(ConnectionTimeoutException);
-PHP_MINIT_FUNCTION(WriteException);
-PHP_MINIT_FUNCTION(BulkWriteException);
+extern void php_phongo_binary_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_decimal128_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_javascript_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_maxkey_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_minkey_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_objectid_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_persistable_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_regex_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_serializable_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_timestamp_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_type_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_utcdatetime_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_unserializable_init_ce(INIT_FUNC_ARGS);
 
-PHP_MINIT_FUNCTION(Type);
-PHP_MINIT_FUNCTION(Unserializable);
-PHP_MINIT_FUNCTION(Serializable);
-PHP_MINIT_FUNCTION(Persistable);
-PHP_MINIT_FUNCTION(Binary);
-PHP_MINIT_FUNCTION(Decimal128);
-PHP_MINIT_FUNCTION(Javascript);
-PHP_MINIT_FUNCTION(MaxKey);
-PHP_MINIT_FUNCTION(MinKey);
-PHP_MINIT_FUNCTION(ObjectID);
-PHP_MINIT_FUNCTION(Regex);
-PHP_MINIT_FUNCTION(Timestamp);
-PHP_MINIT_FUNCTION(UTCDateTime);
+extern void php_phongo_binary_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_decimal128_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_javascript_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_maxkey_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_minkey_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_objectid_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_regex_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_timestamp_interface_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_utcdatetime_interface_init_ce(INIT_FUNC_ARGS);
+
+extern void php_phongo_bulkwrite_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_command_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_cursor_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_cursorid_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_manager_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_query_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_readconcern_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_readpreference_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_server_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_writeconcern_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_writeconcernerror_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_writeerror_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_writeresult_init_ce(INIT_FUNC_ARGS);
+
+extern void php_phongo_authenticationexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_bulkwriteexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_connectionexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_connectiontimeoutexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_exception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_executiontimeoutexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_invalidargumentexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_logicexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_runtimeexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_sslconnectionexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_unexpectedvalueexception_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_writeexception_init_ce(INIT_FUNC_ARGS);
+
+extern void php_phongo_commandfailedevent_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_commandstartedevent_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_commandsubscriber_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_commandsucceededevent_init_ce(INIT_FUNC_ARGS);
+extern void php_phongo_subscriber_init_ce(INIT_FUNC_ARGS);
+
+/* Shared function entries for disabling constructors and unserialize() */
+PHP_FUNCTION(MongoDB_disabled___construct);
+PHP_FUNCTION(MongoDB_disabled___wakeup);
 
 #endif /* PHONGO_CLASSES_H */
 
