@@ -52,7 +52,6 @@ typedef struct {
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
 	mongoc_cursor_t       *cursor;
-	PHONGO_STRUCT_ZVAL     manager;
 	mongoc_client_t       *client;
 	int                    server_id;
 	php_phongo_bson_state  visitor_data;
@@ -68,7 +67,7 @@ typedef struct {
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	uint64_t id;
+	int64_t id;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_cursorid_t;
 
@@ -83,6 +82,7 @@ typedef struct {
 	bson_t                *filter;
 	bson_t                *opts;
 	mongoc_read_concern_t *read_concern;
+	uint32_t               max_await_time_ms;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_query_t;
 
@@ -100,7 +100,6 @@ typedef struct {
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	PHONGO_STRUCT_ZVAL  manager;
 	mongoc_client_t    *client;
 	int                 server_id;
 	PHONGO_ZEND_OBJECT_POST
@@ -133,7 +132,6 @@ typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
 	mongoc_write_concern_t *write_concern;
 	bson_t                 *reply;
-	PHONGO_STRUCT_ZVAL      manager;
 	mongoc_client_t        *client;
 	int                     server_id;
 	PHONGO_ZEND_OBJECT_POST
@@ -141,24 +139,27 @@ typedef struct {
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	char    *data;
-	int      data_len;
-	uint8_t  type;
+	char      *data;
+	int        data_len;
+	uint8_t    type;
+	HashTable *properties;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_binary_t;
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	bool              initialized;
-	bson_decimal128_t decimal;
+	bool               initialized;
+	bson_decimal128_t  decimal;
+	HashTable         *properties;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_decimal128_t;
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	char   *code;
-	size_t  code_len;
-	bson_t *scope;
+	char      *code;
+	size_t     code_len;
+	bson_t    *scope;
+	HashTable *properties;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_javascript_t;
 
@@ -174,35 +175,75 @@ typedef struct {
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	bool initialized;
-	char oid[25];
+	bool       initialized;
+	char       oid[25];
+	HashTable *properties;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_objectid_t;
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	char *pattern;
-	int   pattern_len;
-	char *flags;
-	int   flags_len;
+	char      *pattern;
+	int        pattern_len;
+	char      *flags;
+	int        flags_len;
+	HashTable *properties;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_regex_t;
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	bool     initialized;
-	uint32_t increment;
-	uint32_t timestamp;
+	bool       initialized;
+	uint32_t   increment;
+	uint32_t   timestamp;
+	HashTable *properties;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_timestamp_t;
 
 typedef struct {
 	PHONGO_ZEND_OBJECT_PRE
-	bool    initialized;
-	int64_t milliseconds;
+	bool       initialized;
+	int64_t    milliseconds;
+	HashTable *properties;
 	PHONGO_ZEND_OBJECT_POST
 } php_phongo_utcdatetime_t;
 
+typedef struct {
+	PHONGO_ZEND_OBJECT_PRE
+	mongoc_client_t    *client;
+	char               *command_name;
+	uint32_t            server_id;
+	uint64_t            operation_id;
+	uint64_t            request_id;
+	uint64_t            duration_micros;
+	PHONGO_STRUCT_ZVAL  z_error;
+	PHONGO_ZEND_OBJECT_POST
+} php_phongo_commandfailedevent_t;
+
+typedef struct {
+	PHONGO_ZEND_OBJECT_PRE
+	mongoc_client_t    *client;
+	char               *command_name;
+	uint32_t            server_id;
+	uint64_t            operation_id;
+	uint64_t            request_id;
+	bson_t             *command;
+	char               *database_name;
+	PHONGO_ZEND_OBJECT_POST
+} php_phongo_commandstartedevent_t;
+
+typedef struct {
+	PHONGO_ZEND_OBJECT_PRE
+	mongoc_client_t    *client;
+	char               *command_name;
+	uint32_t            server_id;
+	uint64_t            operation_id;
+	uint64_t            request_id;
+	uint64_t            duration_micros;
+	bson_t             *reply;
+	PHONGO_ZEND_OBJECT_POST
+} php_phongo_commandsucceededevent_t;
+
 #undef PHONGO_ZEND_OBJECT_PRE
 #undef PHONGO_ZEND_OBJECT_POST
 #undef PHONGO_STRUCT_ZVAL
