@@ -1,53 +1,75 @@
 /*
-  +---------------------------------------------------------------------------+
-  | PHP Driver for MongoDB                                                    |
-  +---------------------------------------------------------------------------+
-  | Copyright 2013-2015 MongoDB, Inc.                                         |
-  |                                                                           |
-  | Licensed under the Apache License, Version 2.0 (the "License");           |
-  | you may not use this file except in compliance with the License.          |
-  | You may obtain a copy of the License at                                   |
-  |                                                                           |
-  | http://www.apache.org/licenses/LICENSE-2.0                                |
-  |                                                                           |
-  | Unless required by applicable law or agreed to in writing, software       |
-  | distributed under the License is distributed on an "AS IS" BASIS,         |
-  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  |
-  | See the License for the specific language governing permissions and       |
-  | limitations under the License.                                            |
-  +---------------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 MongoDB, Inc.                                     |
-  +---------------------------------------------------------------------------+
-*/
+ * Copyright 2015-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
-#	include "config.h"
+# include "config.h"
 #endif
 
-/* External libs */
-#include <bson.h>
-#include <mongoc.h>
-
-/* PHP Core stuff */
 #include <php.h>
-#include <php_ini.h>
-#include <ext/standard/info.h>
 #include <Zend/zend_interfaces.h>
 #include <ext/spl/spl_iterators.h>
-/* PHP array helpers */
-#include "php_array_api.h"
-/* Our Compatability header */
-#include "phongo_compat.h"
 
-/* Our stuffz */
+#include "php_array_api.h"
+#include "phongo_compat.h"
 #include "php_phongo.h"
 #include "php_bson.h"
 
-#define BYPASS_UNSET -1
+#define PHONGO_BULKWRITE_BYPASS_UNSET -1
 
-PHONGO_API zend_class_entry *php_phongo_bulkwrite_ce;
+zend_class_entry *php_phongo_bulkwrite_ce;
 
-zend_object_handlers php_phongo_handler_bulkwrite;
+/* Returns whether the insert document appears to be a legacy index. */
+static inline bool php_phongo_bulkwrite_insert_is_legacy_index(bson_t *bdocument) /* {{{ */
+{
+	bson_iter_t iter;
+
+	if (bson_iter_init_find(&iter, bdocument, "key") && BSON_ITER_HOLDS_DOCUMENT(&iter) &&
+	    bson_iter_init_find(&iter, bdocument, "name") && BSON_ITER_HOLDS_UTF8(&iter) &&
+	    bson_iter_init_find(&iter, bdocument, "ns") && BSON_ITER_HOLDS_UTF8(&iter)) {
+		return true;
+	}
+
+	return false;
+} /* }}} */
+
+/* Extracts the "_id" field of a BSON document into a return value. */
+static void php_phongo_bulkwrite_extract_id(bson_t *doc, zval **return_value) /* {{{ */
+{
+	php_phongo_bson_state state = PHONGO_BSON_STATE_INITIALIZER;
+	zval *id = NULL;
+
+	state.map.root_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
+
+	if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &state)) {
+		goto cleanup;
+	}
+
+#if PHP_VERSION_ID >= 70000
+	id = php_array_fetchc(&state.zchild, "_id");
+#else
+	id = php_array_fetchc(state.zchild, "_id");
+#endif
+
+	if (id) {
+		ZVAL_ZVAL(*return_value, id, 1, 0);
+	}
+
+cleanup:
+	zval_ptr_dtor(&state.zchild);
+} /* }}} */
 
 /* Returns whether any top-level field names in the document contain a "$". */
 static inline bool php_phongo_bulkwrite_update_has_operators(bson_t *bupdate) /* {{{ */
@@ -67,7 +89,7 @@ static inline bool php_phongo_bulkwrite_update_has_operators(bson_t *bupdate) /*
 
 /* Appends a document field for the given opts document and key. Returns true on
  * success; otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_bulkwrite_opts_append_document(bson_t *opts, const char *opts_key, zval *zarr, const char *zarr_key TSRMLS_DC)
+static bool php_phongo_bulkwrite_opts_append_document(bson_t *opts, const char *opts_key, zval *zarr, const char *zarr_key TSRMLS_DC) /* {{{ */
 {
 	zval *value = php_array_fetch(zarr, zarr_key);
 	bson_t b = BSON_INITIALIZER;
@@ -77,7 +99,7 @@ static bool php_phongo_bulkwrite_opts_append_document(bson_t *opts, const char *
 		return false;
 	}
 
-	phongo_zval_to_bson(value, PHONGO_BSON_NONE, &b, NULL TSRMLS_CC);
+	php_phongo_zval_to_bson(value, PHONGO_BSON_NONE, &b, NULL TSRMLS_CC);
 
 	if (EG(exception)) {
 		bson_destroy(&b);
@@ -92,7 +114,7 @@ static bool php_phongo_bulkwrite_opts_append_document(bson_t *opts, const char *
 
 	bson_destroy(&b);
 	return true;
-}
+} /* }}} */
 
 #define PHONGO_BULKWRITE_APPEND_BOOL(opt, value) \
 	if (!BSON_APPEND_BOOL(boptions, (opt), (value))) { \
@@ -155,9 +177,9 @@ static bool php_phongo_bulkwrite_delete_apply_options(bson_t *boptions, zval *zo
 #undef PHONGO_BULKWRITE_APPEND_INT32
 #undef PHONGO_BULKWRITE_OPT_DOCUMENT
 
-/* {{{ proto void BulkWrite::__construct([array $options = array()])
+/* {{{ proto void MongoDB\Driver\BulkWrite::__construct([array $options = array()])
    Constructs a new BulkWrite */
-PHP_METHOD(BulkWrite, __construct)
+static PHP_METHOD(BulkWrite, __construct)
 {
 	php_phongo_bulkwrite_t  *intern;
 	zend_error_handling      error_handling;
@@ -181,7 +203,7 @@ PHP_METHOD(BulkWrite, __construct)
 
 	intern->bulk = phongo_bulkwrite_init(ordered);
 	intern->ordered = ordered;
-	intern->bypass = BYPASS_UNSET;
+	intern->bypass = PHONGO_BULKWRITE_BYPASS_UNSET;
 	intern->num_ops = 0;
 
 	if (options && php_array_existsc(options, "bypassDocumentValidation")) {
@@ -189,61 +211,77 @@ PHP_METHOD(BulkWrite, __construct)
 		mongoc_bulk_operation_set_bypass_document_validation(intern->bulk, bypass);
 		intern->bypass = bypass;
 	}
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto mixed BulkWrite::insert(array|object $document)
+/* {{{ proto mixed MongoDB\Driver\BulkWrite::insert(array|object $document)
    Adds an insert operation to the BulkWrite */
-PHP_METHOD(BulkWrite, insert)
+static PHP_METHOD(BulkWrite, insert)
 {
-	php_phongo_bulkwrite_t  *intern;
-	zval                     *document;
-	bson_t                   *bson;
-	bson_t                   *bson_out = NULL;
-	int                       bson_flags = PHONGO_BSON_ADD_ID;
+	php_phongo_bulkwrite_t *intern;
+	zval                   *zdocument;
+	bson_t                  bdocument = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
+	bson_t                 *bson_out = NULL;
+	int                     bson_flags = PHONGO_BSON_ADD_ID;
+	bson_error_t            error = {0};
 	DECLARE_RETURN_VALUE_USED
 	SUPPRESS_UNUSED_WARNING(return_value_ptr)
 
 
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "A", &document) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "A", &zdocument) == FAILURE) {
 		return;
 	}
 
-
 	if (return_value_used) {
 		bson_flags |= PHONGO_BSON_RETURN_ID;
 	}
 
-	bson = bson_new();
-	phongo_zval_to_bson(document, bson_flags, bson, &bson_out TSRMLS_CC);
-	mongoc_bulk_operation_insert(intern->bulk, bson);
-	bson_clear(&bson);
+	php_phongo_zval_to_bson(zdocument, bson_flags, &bdocument, &bson_out TSRMLS_CC);
+
+	if (EG(exception)) {
+		goto cleanup;
+	}
+
+	/* If the insert document appears to be a legacy index, instruct libmongoc
+	 * to allow dots in BSON keys by setting the "legacyIndex" option.
+	 *
+	 * Note: php_phongo_zval_to_bson() may have added an ObjectId if the "_id"
+	 * field was unset. We don't know at this point if the insert is destined
+	 * for a pre-2.6 server's "system.indexes" collection, but legacy index
+	 * creation will ignore the "_id" so there is no harm in leaving it. In the
+	 * event php_phongo_bulkwrite_insert_is_legacy_index() returns a false
+	 * positive, we absolutely want ObjectId added if "_id" was unset. */
+	if (php_phongo_bulkwrite_insert_is_legacy_index(&bdocument) &&
+	    !BSON_APPEND_BOOL(&boptions, "legacyIndex", true)) {
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Error appending \"legacyIndex\" option");
+		goto cleanup;
+	}
+
+	if (!mongoc_bulk_operation_insert_with_opts(intern->bulk, &bdocument, &boptions, &error)) {
+		phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
+		goto cleanup;
+	}
 
 	intern->num_ops++;
 
 	if (bson_out && return_value_used) {
-		bson_iter_t iter;
-
-		if (bson_iter_init_find(&iter, bson_out, "_id")) {
-			php_phongo_objectid_new_from_oid(return_value, bson_iter_oid(&iter) TSRMLS_CC);
-			bson_clear(&bson_out);
-			return;
-		}
-
-		bson_clear(&bson_out);
+		php_phongo_bulkwrite_extract_id(bson_out, &return_value);
 	}
-}
-/* }}} */
 
-/* {{{ proto void BulkWrite::update(array|object $query, array|object $newObj[, array $updateOptions = array()])
+cleanup:
+	bson_destroy(&bdocument);
+	bson_destroy(&boptions);
+	bson_clear(&bson_out);
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\BulkWrite::update(array|object $query, array|object $newObj[, array $updateOptions = array()])
    Adds an update operation to the BulkWrite */
-PHP_METHOD(BulkWrite, update)
+static PHP_METHOD(BulkWrite, update)
 {
 	php_phongo_bulkwrite_t *intern;
 	zval                   *zquery, *zupdate, *zoptions = NULL;
-	bson_t                 *bquery, *bupdate, *boptions = NULL;
+	bson_t                  bquery = BSON_INITIALIZER, bupdate = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
 	bson_error_t            error = {0};
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value) SUPPRESS_UNUSED_WARNING(return_value_used)
 
@@ -254,50 +292,41 @@ PHP_METHOD(BulkWrite, update)
 		return;
 	}
 
-	bquery = bson_new();
-	bupdate = bson_new();
-	boptions = bson_new();
-
-	phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, bquery, NULL TSRMLS_CC);
+	php_phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, &bquery, NULL TSRMLS_CC);
 
 	if (EG(exception)) {
 		goto cleanup;
 	}
 
-	phongo_zval_to_bson(zupdate, PHONGO_BSON_NONE, bupdate, NULL TSRMLS_CC);
+	php_phongo_zval_to_bson(zupdate, PHONGO_BSON_NONE, &bupdate, NULL TSRMLS_CC);
 
 	if (EG(exception)) {
 		goto cleanup;
 	}
 
-	if (!php_phongo_bulkwrite_update_apply_options(boptions, zoptions TSRMLS_CC)) {
+	if (!php_phongo_bulkwrite_update_apply_options(&boptions, zoptions TSRMLS_CC)) {
 		goto cleanup;
 	}
 
-	if (php_phongo_bulkwrite_update_has_operators(bupdate)) {
+	if (php_phongo_bulkwrite_update_has_operators(&bupdate)) {
 		if (zoptions && php_array_existsc(zoptions, "multi") && php_array_fetchc_bool(zoptions, "multi")) {
-			if (!mongoc_bulk_operation_update_many_with_opts(intern->bulk, bquery, bupdate, boptions, &error)) {
+			if (!mongoc_bulk_operation_update_many_with_opts(intern->bulk, &bquery, &bupdate, &boptions, &error)) {
 				phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
 				goto cleanup;
 			}
 		} else {
-			if (!mongoc_bulk_operation_update_one_with_opts(intern->bulk, bquery, bupdate, boptions, &error)) {
+			if (!mongoc_bulk_operation_update_one_with_opts(intern->bulk, &bquery, &bupdate, &boptions, &error)) {
 				phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
 				goto cleanup;
 			}
 		}
 	} else {
-		if (!bson_validate(bupdate, BSON_VALIDATE_DOT_KEYS|BSON_VALIDATE_DOLLAR_KEYS, NULL)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Replacement document may not contain \"$\" or \".\" in keys");
-			goto cleanup;
-		}
-
 		if (zoptions && php_array_existsc(zoptions, "multi") && php_array_fetchc_bool(zoptions, "multi")) {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Replacement document conflicts with true \"multi\" option");
 			goto cleanup;
 		}
 
-		if (!mongoc_bulk_operation_replace_one_with_opts(intern->bulk, bquery, bupdate, boptions, &error)) {
+		if (!mongoc_bulk_operation_replace_one_with_opts(intern->bulk, &bquery, &bupdate, &boptions, &error)) {
 			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
 			goto cleanup;
 		}
@@ -306,19 +335,18 @@ PHP_METHOD(BulkWrite, update)
 	intern->num_ops++;
 
 cleanup:
-	bson_clear(&bquery);
-	bson_clear(&bupdate);
-	bson_clear(&boptions);
-}
-/* }}} */
+	bson_destroy(&bquery);
+	bson_destroy(&bupdate);
+	bson_destroy(&boptions);
+} /* }}} */
 
-/* {{{ proto void BulkWrite::delete(array|object $query[, array $deleteOptions = array()])
+/* {{{ proto void MongoDB\Driver\BulkWrite::delete(array|object $query[, array $deleteOptions = array()])
    Adds a delete operation to the BulkWrite */
-PHP_METHOD(BulkWrite, delete)
+static PHP_METHOD(BulkWrite, delete)
 {
 	php_phongo_bulkwrite_t *intern;
 	zval                   *zquery, *zoptions = NULL;
-	bson_t                 *bquery, *boptions = NULL;
+	bson_t                  bquery = BSON_INITIALIZER, boptions = BSON_INITIALIZER;
 	bson_error_t            error = {0};
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value) SUPPRESS_UNUSED_WARNING(return_value_used)
 
@@ -329,26 +357,23 @@ PHP_METHOD(BulkWrite, delete)
 		return;
 	}
 
-	bquery = bson_new();
-	boptions = bson_new();
-
-	phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, bquery, NULL TSRMLS_CC);
+	php_phongo_zval_to_bson(zquery, PHONGO_BSON_NONE, &bquery, NULL TSRMLS_CC);
 
 	if (EG(exception)) {
 		goto cleanup;
 	}
 
-	if (!php_phongo_bulkwrite_delete_apply_options(boptions, zoptions TSRMLS_CC)) {
+	if (!php_phongo_bulkwrite_delete_apply_options(&boptions, zoptions TSRMLS_CC)) {
 		goto cleanup;
 	}
 
 	if (zoptions && php_array_existsc(zoptions, "limit") && php_array_fetchc_bool(zoptions, "limit")) {
-		if (!mongoc_bulk_operation_remove_one_with_opts(intern->bulk, bquery, boptions, &error)) {
+		if (!mongoc_bulk_operation_remove_one_with_opts(intern->bulk, &bquery, &boptions, &error)) {
 			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
 			goto cleanup;
 		}
 	} else {
-		if (!mongoc_bulk_operation_remove_many_with_opts(intern->bulk, bquery, boptions, &error)) {
+		if (!mongoc_bulk_operation_remove_many_with_opts(intern->bulk, &bquery, &boptions, &error)) {
 			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
 			goto cleanup;
 		}
@@ -357,14 +382,13 @@ PHP_METHOD(BulkWrite, delete)
 	intern->num_ops++;
 
 cleanup:
-	bson_clear(&bquery);
-	bson_clear(&boptions);
-}
-/* }}} */
+	bson_destroy(&bquery);
+	bson_destroy(&boptions);
+} /* }}} */
 
-/* {{{ proto integer BulkWrite::count()
+/* {{{ proto integer MongoDB\Driver\BulkWrite::count()
    Returns the number of operations that have been added to the BulkWrite */
-PHP_METHOD(BulkWrite, count)
+static PHP_METHOD(BulkWrite, count)
 {
 	php_phongo_bulkwrite_t  *intern;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value) SUPPRESS_UNUSED_WARNING(return_value_used)
@@ -377,12 +401,9 @@ PHP_METHOD(BulkWrite, count)
 	}
 
 	RETURN_LONG(intern->num_ops);
-}
-/* }}} */
-
-
-/* {{{ MongoDB\Driver\BulkWrite */
+} /* }}} */
 
+/* {{{ MongoDB\Driver\BulkWrite function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite___construct, 0, 0, 0)
 	ZEND_ARG_ARRAY_INFO(0, options, 1)
 ZEND_END_ARG_INFO()
@@ -411,14 +432,14 @@ static zend_function_entry php_phongo_bulkwrite_me[] = {
 	PHP_ME(BulkWrite, update, ai_BulkWrite_update, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, delete, ai_BulkWrite_delete, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, count, ai_BulkWrite_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Manager, __wakeup, ai_BulkWrite_void, ZEND_ACC_PUBLIC)
+	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_BulkWrite_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_FE_END
 };
-
 /* }}} */
 
+/* {{{ MongoDB\Driver\BulkWrite object handlers */
+static zend_object_handlers php_phongo_handler_bulkwrite;
 
-/* {{{ php_phongo_bulkwrite_t object handlers */
 static void php_phongo_bulkwrite_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
 {
 	php_phongo_bulkwrite_t *intern = Z_OBJ_BULKWRITE(object);
@@ -442,7 +463,7 @@ static void php_phongo_bulkwrite_free_object(phongo_free_object_arg *object TSRM
 #endif
 } /* }}} */
 
-phongo_create_object_retval php_phongo_bulkwrite_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static phongo_create_object_retval php_phongo_bulkwrite_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
 {
 	php_phongo_bulkwrite_t *intern = NULL;
 
@@ -466,7 +487,7 @@ phongo_create_object_retval php_phongo_bulkwrite_create_object(zend_class_entry
 #endif
 } /* }}} */
 
-HashTable *php_phongo_bulkwrite_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+static HashTable *php_phongo_bulkwrite_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
 {
 #if PHP_VERSION_ID >= 70000
 	zval                      retval;
@@ -494,7 +515,7 @@ HashTable *php_phongo_bulkwrite_get_debug_info(zval *object, int *is_temp TSRMLS
 
 	ADD_ASSOC_BOOL_EX(&retval, "ordered", intern->ordered);
 
-	if (intern->bypass != BYPASS_UNSET) {
+	if (intern->bypass != PHONGO_BULKWRITE_BYPASS_UNSET) {
 		ADD_ASSOC_BOOL_EX(&retval, "bypassDocumentValidation", intern->bypass);
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "bypassDocumentValidation");
@@ -520,17 +541,13 @@ HashTable *php_phongo_bulkwrite_get_debug_info(zval *object, int *is_temp TSRMLS
 		ADD_ASSOC_NULL_EX(&retval, "write_concern");
 	}
 
-
-
 	return Z_ARRVAL(retval);
 } /* }}} */
 /* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION */
-PHP_MINIT_FUNCTION(BulkWrite)
+void php_phongo_bulkwrite_init_ce(INIT_FUNC_ARGS) /* {{{ */
 {
 	zend_class_entry ce;
-	(void)type;(void)module_number;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "BulkWrite", php_phongo_bulkwrite_me);
 	php_phongo_bulkwrite_ce = zend_register_internal_class(&ce TSRMLS_CC);
@@ -546,13 +563,7 @@ PHP_MINIT_FUNCTION(BulkWrite)
 #endif
 
 	zend_class_implements(php_phongo_bulkwrite_ce TSRMLS_CC, 1, spl_ce_Countable);
-
-
-	return SUCCESS;
-}
-/* }}} */
-
-
+} /* }}} */
 
 /*
  * Local variables:
