@@ -1,54 +1,34 @@
 /*
-  +---------------------------------------------------------------------------+
-  | PHP Driver for MongoDB                                                    |
-  +---------------------------------------------------------------------------+
-  | Copyright 2013-2015 MongoDB, Inc.                                         |
-  |                                                                           |
-  | Licensed under the Apache License, Version 2.0 (the "License");           |
-  | you may not use this file except in compliance with the License.          |
-  | You may obtain a copy of the License at                                   |
-  |                                                                           |
-  | http://www.apache.org/licenses/LICENSE-2.0                                |
-  |                                                                           |
-  | Unless required by applicable law or agreed to in writing, software       |
-  | distributed under the License is distributed on an "AS IS" BASIS,         |
-  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  |
-  | See the License for the specific language governing permissions and       |
-  | limitations under the License.                                            |
-  +---------------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 MongoDB, Inc.                                     |
-  +---------------------------------------------------------------------------+
-*/
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
-#	include "config.h"
+# include "config.h"
 #endif
 
-/* External libs */
-#include <bson.h>
-#include <mongoc.h>
-
-/* PHP Core stuff */
 #include <php.h>
-#include <php_ini.h>
-#include <ext/standard/info.h>
 #include <Zend/zend_interfaces.h>
-#include <ext/spl/spl_iterators.h>
-/* Our Compatability header */
+
 #include "phongo_compat.h"
-
-/* Our stuffz */
 #include "php_phongo.h"
-#include "php_bson.h"
 
+zend_class_entry *php_phongo_cursorid_ce;
 
-PHONGO_API zend_class_entry *php_phongo_cursorid_ce;
-
-zend_object_handlers php_phongo_handler_cursorid;
-
-/* {{{ proto string CursorId::__toString()
+/* {{{ proto string MongoDB\Driver\CursorId::__toString()
    Returns the string representation of the CursorId */
-PHP_METHOD(CursorId, __toString)
+static PHP_METHOD(CursorId, __toString)
 {
 	php_phongo_cursorid_t    *intern;
 	char *tmp;
@@ -61,28 +41,26 @@ PHP_METHOD(CursorId, __toString)
 		return;
 	}
 
-	tmp_len = spprintf(&tmp, 0, "%" PRIu64, intern->id);
+	tmp_len = spprintf(&tmp, 0, "%" PRId64, intern->id);
 	PHONGO_RETVAL_STRINGL(tmp, tmp_len);
 	efree(tmp);
-}
-/* }}} */
-
-/* {{{ MongoDB\Driver\CursorId */
+} /* }}} */
 
+/* {{{ MongoDB\Driver\CursorId function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_CursorId_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_cursorid_me[] = {
-	PHP_ME(Server, __construct, ai_CursorId_void, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
 	PHP_ME(CursorId, __toString, ai_CursorId_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Manager, __wakeup, ai_CursorId_void, ZEND_ACC_PUBLIC)
+	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_CursorId_void, ZEND_ACC_PRIVATE|ZEND_ACC_FINAL)
+	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_CursorId_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_FE_END
 };
-
 /* }}} */
 
+/* {{{ MongoDB\Driver\CursorId object handlers */
+static zend_object_handlers php_phongo_handler_cursorid;
 
-/* {{{ php_phongo_cursorid_t object handlers */
 static void php_phongo_cursorid_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
 {
 	php_phongo_cursorid_t *intern = Z_OBJ_CURSORID(object);
@@ -94,7 +72,7 @@ static void php_phongo_cursorid_free_object(phongo_free_object_arg *object TSRML
 #endif
 } /* }}} */
 
-phongo_create_object_retval php_phongo_cursorid_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static phongo_create_object_retval php_phongo_cursorid_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
 {
 	php_phongo_cursorid_t *intern = NULL;
 
@@ -118,7 +96,7 @@ phongo_create_object_retval php_phongo_cursorid_create_object(zend_class_entry *
 #endif
 } /* }}} */
 
-HashTable *php_phongo_cursorid_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+static HashTable *php_phongo_cursorid_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
 {
 	php_phongo_cursorid_t  *intern;
 #if PHP_VERSION_ID >= 70000
@@ -138,7 +116,7 @@ HashTable *php_phongo_cursorid_get_debug_info(zval *object, int *is_temp TSRMLS_
 		char tmp[24];
 		int tmp_len;
 
-		tmp_len = snprintf(tmp, sizeof(tmp), "%" PRIu64, intern->id);
+		tmp_len = snprintf(tmp, sizeof(tmp), "%" PRId64, intern->id);
 		ADD_ASSOC_STRINGL(&retval, "id", tmp, tmp_len);
 	}
 #else
@@ -149,11 +127,9 @@ HashTable *php_phongo_cursorid_get_debug_info(zval *object, int *is_temp TSRMLS_
 } /* }}} */
 /* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION */
-PHP_MINIT_FUNCTION(CursorId)
+void php_phongo_cursorid_init_ce(INIT_FUNC_ARGS) /* {{{ */
 {
 	zend_class_entry ce;
-	(void)type;(void)module_number;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "CursorId", php_phongo_cursorid_me);
 	php_phongo_cursorid_ce = zend_register_internal_class(&ce TSRMLS_CC);
@@ -167,12 +143,7 @@ PHP_MINIT_FUNCTION(CursorId)
 	php_phongo_handler_cursorid.free_obj = php_phongo_cursorid_free_object;
 	php_phongo_handler_cursorid.offset = XtOffsetOf(php_phongo_cursorid_t, std);
 #endif
-
-	return SUCCESS;
-}
-/* }}} */
-
-
+} /* }}} */
 
 /*
  * Local variables:
