@@ -15,38 +15,22 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#	include "config.h"
+# include "config.h"
 #endif
 
-/* External libs */
-#include <bson.h>
-#include <mongoc.h>
-
-/* PHP Core stuff */
 #include <php.h>
-#include <php_ini.h>
-#include <ext/standard/info.h>
 #include <Zend/zend_interfaces.h>
-#include <ext/spl/spl_iterators.h>
 
-/* PHP array helpers */
 #include "php_array_api.h"
-
-/* Our Compatability header */
 #include "phongo_compat.h"
-
-/* Our stuffz */
 #include "php_phongo.h"
 #include "php_bson.h"
 
+zend_class_entry *php_phongo_query_ce;
 
-PHONGO_API zend_class_entry *php_phongo_query_ce;
-
-zend_object_handlers php_phongo_handler_query;
-
-/* Appends a string field into the BSON options. Returns true on
- * success; otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_query_opts_append_string(bson_t *opts, const char *opts_key, zval *zarr, const char *zarr_key TSRMLS_DC)
+/* Appends a string field into the BSON options. Returns true on success;
+ * otherwise, false is returned and an exception is thrown. */
+static bool php_phongo_query_opts_append_string(bson_t *opts, const char *opts_key, zval *zarr, const char *zarr_key TSRMLS_DC) /* {{{ */
 {
 	zval *value = php_array_fetch(zarr, zarr_key);
 
@@ -61,11 +45,11 @@ static bool php_phongo_query_opts_append_string(bson_t *opts, const char *opts_k
 	}
 
 	return true;
-}
+} /* }}} */
 
 /* Appends a document field for the given opts document and key. Returns true on
  * success; otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_query_opts_append_document(bson_t *opts, const char *opts_key, zval *zarr, const char *zarr_key TSRMLS_DC)
+static bool php_phongo_query_opts_append_document(bson_t *opts, const char *opts_key, zval *zarr, const char *zarr_key TSRMLS_DC) /* {{{ */
 {
 	zval *value = php_array_fetch(zarr, zarr_key);
 	bson_t b = BSON_INITIALIZER;
@@ -75,13 +59,19 @@ static bool php_phongo_query_opts_append_document(bson_t *opts, const char *opts
 		return false;
 	}
 
-	phongo_zval_to_bson(value, PHONGO_BSON_NONE, &b, NULL TSRMLS_CC);
+	php_phongo_zval_to_bson(value, PHONGO_BSON_NONE, &b, NULL TSRMLS_CC);
 
 	if (EG(exception)) {
 		bson_destroy(&b);
 		return false;
 	}
 
+	if (!bson_validate(&b, BSON_VALIDATE_EMPTY_KEYS, NULL)) {
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Cannot use empty keys in \"%s\" %s", zarr_key, zarr_key[0] == '$' ? "modifier" : "option");
+		bson_destroy(&b);
+		return false;
+	}
+
 	if (!BSON_APPEND_DOCUMENT(opts, opts_key, &b)) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Error appending \"%s\" option", opts_key);
 		bson_destroy(&b);
@@ -90,7 +80,7 @@ static bool php_phongo_query_opts_append_document(bson_t *opts, const char *opts
 
 	bson_destroy(&b);
 	return true;
-}
+} /* }}} */
 
 #define PHONGO_QUERY_OPT_BOOL(opt, zarr, key) \
 	if ((zarr) && php_array_existsc((zarr), (key))) { \
@@ -215,6 +205,32 @@ static bool php_phongo_query_init_readconcern(php_phongo_query_t *intern, zval *
 	return true;
 } /* }}} */
 
+/* Initialize the "maxAwaitTimeMS" option. Returns true on success; otherwise,
+ * false is returned and an exception is thrown.
+ *
+ * The "maxAwaitTimeMS" option is assigned to the cursor after query execution
+ * via mongoc_cursor_set_max_await_time_ms(). */
+static bool php_phongo_query_init_max_await_time_ms(php_phongo_query_t *intern, zval *options TSRMLS_DC) /* {{{ */
+{
+	if (php_array_existsc(options, "maxAwaitTimeMS")) {
+		int64_t max_await_time_ms = php_array_fetchc_long(options, "maxAwaitTimeMS");
+
+		if (max_await_time_ms < 0) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected \"maxAwaitTimeMS\" option to be >= 0, %" PRId64 " given", max_await_time_ms);
+			return false;
+		}
+
+		if (max_await_time_ms > UINT32_MAX) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected \"maxAwaitTimeMS\" option to be <= %" PRIu32 ", %" PRId64 " given", UINT32_MAX, max_await_time_ms);
+			return false;
+		}
+
+		intern->max_await_time_ms = (uint32_t) max_await_time_ms;
+	}
+
+	return true;
+} /* }}} */
+
 /* Initializes the php_phongo_query_t from filter and options arguments. This
  * function will fall back to a modifier in the absence of a top-level option
  * (where applicable). */
@@ -225,7 +241,7 @@ static bool php_phongo_query_init(php_phongo_query_t *intern, zval *filter, zval
 	intern->filter = bson_new();
 	intern->opts = bson_new();
 
-	phongo_zval_to_bson(filter, PHONGO_BSON_NONE, intern->filter, NULL TSRMLS_CC);
+	php_phongo_zval_to_bson(filter, PHONGO_BSON_NONE, intern->filter, NULL TSRMLS_CC);
 
 	/* Note: if any exceptions are thrown, we can simply return as PHP will
 	 * invoke php_phongo_query_free_object to destruct the object. */
@@ -233,6 +249,11 @@ static bool php_phongo_query_init(php_phongo_query_t *intern, zval *filter, zval
 		return false;
 	}
 
+	if (!bson_validate(intern->filter, BSON_VALIDATE_EMPTY_KEYS, NULL)) {
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Cannot use empty keys in filter document");
+		return false;
+	}
+
 	if (!options) {
 		return true;
 	}
@@ -293,6 +314,10 @@ static bool php_phongo_query_init(php_phongo_query_t *intern, zval *filter, zval
 		return false;
 	}
 
+	if (!php_phongo_query_init_max_await_time_ms(intern, options TSRMLS_CC)) {
+		return false;
+	}
+
 	return true;
 } /* }}} */
 
@@ -301,9 +326,9 @@ static bool php_phongo_query_init(php_phongo_query_t *intern, zval *filter, zval
 #undef PHONGO_QUERY_OPT_INT64
 #undef PHONGO_QUERY_OPT_STRING
 
-/* {{{ proto void Query::__construct(array|object $filter[, array $options = array()])
+/* {{{ proto void MongoDB\Driver\Query::__construct(array|object $filter[, array $options = array()])
    Constructs a new Query */
-PHP_METHOD(Query, __construct)
+static PHP_METHOD(Query, __construct)
 {
 	php_phongo_query_t  *intern;
 	zend_error_handling  error_handling;
@@ -322,11 +347,9 @@ PHP_METHOD(Query, __construct)
 	zend_restore_error_handling(&error_handling TSRMLS_CC);
 
 	php_phongo_query_init(intern, filter, options TSRMLS_CC);
-}
-/* }}} */
-
-/* {{{ MongoDB\Driver\Query */
+} /* }}} */
 
+/* {{{ MongoDB\Driver\Query function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_Query___construct, 0, 0, 1)
 	ZEND_ARG_INFO(0, filter)
 	ZEND_ARG_ARRAY_INFO(0, options, 1)
@@ -337,14 +360,14 @@ ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_query_me[] = {
 	PHP_ME(Query, __construct, ai_Query___construct, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Manager, __wakeup, ai_Query_void, ZEND_ACC_PUBLIC)
+	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Query_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_FE_END
 };
-
 /* }}} */
 
+/* {{{ MongoDB\Driver\Query object handlers */
+static zend_object_handlers php_phongo_handler_query;
 
-/* {{{ php_phongo_query_t object handlers */
 static void php_phongo_query_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
 {
 	php_phongo_query_t *intern = Z_OBJ_QUERY(object);
@@ -368,7 +391,7 @@ static void php_phongo_query_free_object(phongo_free_object_arg *object TSRMLS_D
 #endif
 } /* }}} */
 
-phongo_create_object_retval php_phongo_query_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static phongo_create_object_retval php_phongo_query_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
 {
 	php_phongo_query_t *intern = NULL;
 
@@ -392,7 +415,7 @@ phongo_create_object_retval php_phongo_query_create_object(zend_class_entry *cla
 #endif
 } /* }}} */
 
-HashTable *php_phongo_query_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+static HashTable *php_phongo_query_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
 {
 	php_phongo_query_t    *intern;
 #if PHP_VERSION_ID >= 70000
@@ -416,7 +439,7 @@ HashTable *php_phongo_query_get_debug_info(zval *object, int *is_temp TSRMLS_DC)
 		zval *zv;
 #endif
 
-		phongo_bson_to_zval(bson_get_data(intern->filter), intern->filter->len, &zv);
+		php_phongo_bson_to_zval(bson_get_data(intern->filter), intern->filter->len, &zv);
 #if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "filter", &zv);
 #else
@@ -433,7 +456,7 @@ HashTable *php_phongo_query_get_debug_info(zval *object, int *is_temp TSRMLS_DC)
 		zval *zv;
 #endif
 
-		phongo_bson_to_zval(bson_get_data(intern->opts), intern->opts->len, &zv);
+		php_phongo_bson_to_zval(bson_get_data(intern->opts), intern->opts->len, &zv);
 #if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "options", &zv);
 #else
@@ -465,11 +488,9 @@ HashTable *php_phongo_query_get_debug_info(zval *object, int *is_temp TSRMLS_DC)
 } /* }}} */
 /* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION */
-PHP_MINIT_FUNCTION(Query)
+void php_phongo_query_init_ce(INIT_FUNC_ARGS) /* {{{ */
 {
 	zend_class_entry ce;
-	(void)type;(void)module_number;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Query", php_phongo_query_me);
 	php_phongo_query_ce = zend_register_internal_class(&ce TSRMLS_CC);
@@ -483,12 +504,7 @@ PHP_MINIT_FUNCTION(Query)
 	php_phongo_handler_query.free_obj = php_phongo_query_free_object;
 	php_phongo_handler_query.offset = XtOffsetOf(php_phongo_query_t, std);
 #endif
-
-	return SUCCESS;
-}
-/* }}} */
-
-
+} /* }}} */
 
 /*
  * Local variables:
