@@ -1,54 +1,34 @@
 /*
-  +---------------------------------------------------------------------------+
-  | PHP Driver for MongoDB                                                    |
-  +---------------------------------------------------------------------------+
-  | Copyright 2013-2015 MongoDB, Inc.                                         |
-  |                                                                           |
-  | Licensed under the Apache License, Version 2.0 (the "License");           |
-  | you may not use this file except in compliance with the License.          |
-  | You may obtain a copy of the License at                                   |
-  |                                                                           |
-  | http://www.apache.org/licenses/LICENSE-2.0                                |
-  |                                                                           |
-  | Unless required by applicable law or agreed to in writing, software       |
-  | distributed under the License is distributed on an "AS IS" BASIS,         |
-  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  |
-  | See the License for the specific language governing permissions and       |
-  | limitations under the License.                                            |
-  +---------------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 MongoDB, Inc.                                     |
-  +---------------------------------------------------------------------------+
-*/
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
-#	include "config.h"
+# include "config.h"
 #endif
 
-/* External libs */
-#include <bson.h>
-#include <mongoc.h>
-
-/* PHP Core stuff */
 #include <php.h>
-#include <php_ini.h>
-#include <ext/standard/info.h>
 #include <Zend/zend_interfaces.h>
-#include <ext/spl/spl_iterators.h>
-/* Our Compatability header */
+
 #include "phongo_compat.h"
-
-/* Our stuffz */
 #include "php_phongo.h"
-#include "php_bson.h"
 
+zend_class_entry *php_phongo_writeconcernerror_ce;
 
-PHONGO_API zend_class_entry *php_phongo_writeconcernerror_ce;
-
-zend_object_handlers php_phongo_handler_writeconcernerror;
-
-/* {{{ proto integer WriteConcernError::getCode()
+/* {{{ proto integer MongoDB\Driver\WriteConcernError::getCode()
    Returns the MongoDB error code */
-PHP_METHOD(WriteConcernError, getCode)
+static PHP_METHOD(WriteConcernError, getCode)
 {
 	php_phongo_writeconcernerror_t *intern;
 
@@ -61,12 +41,11 @@ PHP_METHOD(WriteConcernError, getCode)
 
 
 	RETURN_LONG(intern->code);
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto mixed WriteConcernError::getInfo()
+/* {{{ proto mixed MongoDB\Driver\WriteConcernError::getInfo()
    Returns additional metadata for the error */
-PHP_METHOD(WriteConcernError, getInfo)
+static PHP_METHOD(WriteConcernError, getInfo)
 {
 	php_phongo_writeconcernerror_t *intern;
 
@@ -85,12 +64,11 @@ PHP_METHOD(WriteConcernError, getInfo)
 		RETURN_ZVAL(intern->info, 1, 0);
 #endif
 	}
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto string WriteConcernError::getMessage()
+/* {{{ proto string MongoDB\Driver\WriteConcernError::getMessage()
    Returns the actual error message from the server */
-PHP_METHOD(WriteConcernError, getMessage)
+static PHP_METHOD(WriteConcernError, getMessage)
 {
 	php_phongo_writeconcernerror_t *intern;
 
@@ -102,30 +80,25 @@ PHP_METHOD(WriteConcernError, getMessage)
 	}
 
 	PHONGO_RETURN_STRING(intern->message);
-}
-/* }}} */
-
-/**
- * Value object for a write concern error.
- */
-/* {{{ MongoDB\Driver\WriteConcernError */
+} /* }}} */
 
+/* {{{ MongoDB\Driver\WriteConcernError function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcernError_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_writeconcernerror_me[] = {
-	PHP_ME(Server, __construct, ai_WriteConcernError_void, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
 	PHP_ME(WriteConcernError, getCode, ai_WriteConcernError_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(WriteConcernError, getInfo, ai_WriteConcernError_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(WriteConcernError, getMessage, ai_WriteConcernError_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Manager, __wakeup, ai_WriteConcernError_void, ZEND_ACC_PUBLIC)
+	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_WriteConcernError_void, ZEND_ACC_PRIVATE|ZEND_ACC_FINAL)
+	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_WriteConcernError_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_FE_END
 };
-
 /* }}} */
 
+/* {{{ MongoDB\Driver\WriteConcernError object handlers */
+static zend_object_handlers php_phongo_handler_writeconcernerror;
 
-/* {{{ php_phongo_writeconcernerror_t object handlers */
 static void php_phongo_writeconcernerror_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
 {
 	php_phongo_writeconcernerror_t *intern = Z_OBJ_WRITECONCERNERROR(object);
@@ -145,7 +118,7 @@ static void php_phongo_writeconcernerror_free_object(phongo_free_object_arg *obj
 #endif
 } /* }}} */
 
-phongo_create_object_retval php_phongo_writeconcernerror_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static phongo_create_object_retval php_phongo_writeconcernerror_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
 {
 	php_phongo_writeconcernerror_t *intern = NULL;
 
@@ -169,7 +142,7 @@ phongo_create_object_retval php_phongo_writeconcernerror_create_object(zend_clas
 #endif
 } /* }}} */
 
-HashTable *php_phongo_writeconcernerror_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+static HashTable *php_phongo_writeconcernerror_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
 {
 	php_phongo_writeconcernerror_t *intern;
 #if PHP_VERSION_ID >= 70000
@@ -201,11 +174,9 @@ HashTable *php_phongo_writeconcernerror_get_debug_info(zval *object, int *is_tem
 } /* }}} */
 /* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION */
-PHP_MINIT_FUNCTION(WriteConcernError)
+void php_phongo_writeconcernerror_init_ce(INIT_FUNC_ARGS) /* {{{ */
 {
 	zend_class_entry ce;
-	(void)type;(void)module_number;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "WriteConcernError", php_phongo_writeconcernerror_me);
 	php_phongo_writeconcernerror_ce = zend_register_internal_class(&ce TSRMLS_CC);
@@ -219,12 +190,7 @@ PHP_MINIT_FUNCTION(WriteConcernError)
 	php_phongo_handler_writeconcernerror.free_obj = php_phongo_writeconcernerror_free_object;
 	php_phongo_handler_writeconcernerror.offset = XtOffsetOf(php_phongo_writeconcernerror_t, std);
 #endif
-
-	return SUCCESS;
-}
-/* }}} */
-
-
+} /* }}} */
 
 /*
  * Local variables:
