@@ -79,6 +79,15 @@
 #endif
 
 
+/*
+ * Define to 1 if you have gmtime_r available on your platform.
+ */
+#define BSON_HAVE_GMTIME_R 1
+#if BSON_HAVE_GMTIME_R != 1
+# undef BSON_HAVE_GMTIME_R
+#endif
+
+
 /*
  * Define to 1 if you have reallocf available on your platform.
  */
@@ -88,14 +97,6 @@
 #endif
 
 
-/*
- * Define to 1 if you have _set_output_format (VS2013 and older).
- */
-#define BSON_NEEDS_SET_OUTPUT_FORMAT 0
-#if BSON_NEEDS_SET_OUTPUT_FORMAT != 1
-# undef BSON_NEEDS_SET_OUTPUT_FORMAT
-#endif
-
 /*
  * Define to 1 if you have struct timespec available on your platform.
  */
