@@ -136,6 +136,16 @@
 #endif
 
 
+/*
+ * Use ASN1_STRING_get0_data () rather than the deprecated ASN1_STRING_data
+ */
+#define MONGOC_HAVE_ASN1_STRING_GET0_DATA 0
+
+#if MONGOC_HAVE_ASN1_STRING_GET0_DATA != 1
+#  undef MONGOC_HAVE_ASN1_STRING_GET0_DATA
+#endif
+
+
 /*
  * MONGOC_ENABLE_SASL is set from configure to determine if we are
  * compiled with SASL support.
@@ -147,6 +157,39 @@
 #endif
 
 
+/*
+ * MONGOC_ENABLE_SASL_CYRUS is set from configure to determine if we are
+ * compiled with Cyrus SASL support.
+ */
+#define MONGOC_ENABLE_SASL_CYRUS 1
+
+#if MONGOC_ENABLE_SASL_CYRUS != 1
+#  undef MONGOC_ENABLE_SASL_CYRUS
+#endif
+
+
+/*
+ * MONGOC_ENABLE_SASL_SSPI is set from configure to determine if we are
+ * compiled with SSPI support.
+ */
+#define MONGOC_ENABLE_SASL_SSPI 0
+
+#if MONGOC_ENABLE_SASL_SSPI != 1
+#  undef MONGOC_ENABLE_SASL_SSPI
+#endif
+
+
+/*
+ * MONGOC_ENABLE_SASL_GSSAPI is set from configure to determine if we are
+ * compiled with GSSAPI support.
+ */
+#define MONGOC_ENABLE_SASL_GSSAPI 0
+
+#if MONGOC_ENABLE_SASL_GSSAPI != 1
+#  undef MONGOC_ENABLE_SASL_GSSAPI
+#endif
+
+
 /*
  * MONGOC_HAVE_SASL_CLIENT_DONE is set from configure to determine if we
  * have SASL and its version is new enough to use sasl_client_done (),
@@ -182,6 +225,56 @@
 #  undef MONGOC_NO_AUTOMATIC_GLOBALS
 #endif
 
+/*
+ * MONGOC_HAVE_SOCKLEN is set from configure to determine if we
+ * need to emulate the type.
+ */
+#define MONGOC_HAVE_SOCKLEN 1
+
+#if MONGOC_HAVE_SOCKLEN != 1
+#  undef MONGOC_HAVE_SOCKLEN
+#endif
+
+
+/*
+ * Set from configure, see
+ * https://curl.haxx.se/mail/lib-2009-04/0287.html
+ */
+#define MONGOC_SOCKET_ARG2 struct sockaddr
+#define MONGOC_SOCKET_ARG3 socklen_t
+
+/*
+ * Enable wire protocol compression negotiation
+ *
+ */
+#define MONGOC_ENABLE_COMPRESSION 0
+
+#if MONGOC_ENABLE_COMPRESSION != 1
+#  undef MONGOC_ENABLE_COMPRESSION
+#endif
+
+/*
+ * Set if we have snappy compression support
+ *
+ */
+#define MONGOC_ENABLE_COMPRESSION_SNAPPY 0
+
+#if MONGOC_ENABLE_COMPRESSION_SNAPPY != 1
+#  undef MONGOC_ENABLE_COMPRESSION_SNAPPY
+#endif
+
+
+/*
+ * Set if we have zlib compression support
+ *
+ */
+#define MONGOC_ENABLE_COMPRESSION_ZLIB 0
+
+#if MONGOC_ENABLE_COMPRESSION_ZLIB != 1
+#  undef MONGOC_ENABLE_COMPRESSION_ZLIB
+#endif
+
+
 /*
  * NOTICE:
  * If you're about to update this file and add a config flag, make sure to
