@@ -17,7 +17,7 @@
 #ifndef MONGOC_LIST_H
 #define MONGOC_LIST_H
 
-#if !defined (MONGOC_COMPILATION)
+#if !defined(MONGOC_COMPILATION)
 #error "Only <mongoc.h> can be included directly."
 #endif
 
@@ -30,23 +30,24 @@ BSON_BEGIN_DECLS
 typedef struct _mongoc_list_t mongoc_list_t;
 
 
-struct _mongoc_list_t
-{
+struct _mongoc_list_t {
    mongoc_list_t *next;
-   void          *data;
+   void *data;
 };
 
 
-mongoc_list_t *_mongoc_list_append  (mongoc_list_t *list,
-                                     void          *data);
-mongoc_list_t *_mongoc_list_prepend (mongoc_list_t *list,
-                                     void          *data);
-mongoc_list_t *_mongoc_list_remove  (mongoc_list_t *list,
-                                     void          *data);
-void           _mongoc_list_foreach (mongoc_list_t *list,
-                                     void (*func) (void *data, void *user_data),
-                                     void *         user_data);
-void           _mongoc_list_destroy (mongoc_list_t *list);
+mongoc_list_t *
+_mongoc_list_append (mongoc_list_t *list, void *data);
+mongoc_list_t *
+_mongoc_list_prepend (mongoc_list_t *list, void *data);
+mongoc_list_t *
+_mongoc_list_remove (mongoc_list_t *list, void *data);
+void
+_mongoc_list_foreach (mongoc_list_t *list,
+                      void (*func) (void *data, void *user_data),
+                      void *user_data);
+void
+_mongoc_list_destroy (mongoc_list_t *list);
 
 
 BSON_END_DECLS
