@@ -24,13 +24,48 @@
 #include "mongoc-host-list.h"
 #include "mongoc-socket-private.h"
 #include "mongoc-trace-private.h"
+#ifdef _WIN32
+#include <Mstcpip.h>
+#endif
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "socket"
 
 
 #define OPERATION_EXPIRED(expire_at) \
-   ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time())))
+   ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time ())))
+
+
+/* either struct sockaddr or void, depending on platform */
+typedef MONGOC_SOCKET_ARG2 mongoc_sockaddr_t;
+
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * _mongoc_socket_capture_errno --
+ *
+ *       Save the errno state for contextual use.
+ *
+ * Returns:
+ *       None.
+ *
+ * Side effects:
+ *       None.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+static void
+_mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
+{
+#ifdef _WIN32
+   errno = sock->errno_ = WSAGetLastError ();
+#else
+   sock->errno_ = errno;
+#endif
+   TRACE ("setting errno: %d %s", sock->errno_, strerror (sock->errno_));
+}
 
 
 /*
@@ -93,16 +128,15 @@ _mongoc_socket_setnonblock (int sd)
  */
 
 static bool
-#ifdef _WIN32
-_mongoc_socket_wait (SOCKET   sd,           /* IN */
-#else
-_mongoc_socket_wait (int      sd,           /* IN */
-#endif
-                     int      events,       /* IN */
-                     int64_t  expire_at)    /* IN */
+_mongoc_socket_wait (mongoc_socket_t *sock, /* IN */
+                     int events,            /* IN */
+                     int64_t expire_at)     /* IN */
 {
 #ifdef _WIN32
-   WSAPOLLFD pfd;
+   fd_set read_fds;
+   fd_set write_fds;
+   fd_set error_fds;
+   struct timeval timeout_tv;
 #else
    struct pollfd pfd;
 #endif
@@ -112,17 +146,29 @@ _mongoc_socket_wait (int      sd,           /* IN */
 
    ENTRY;
 
+   BSON_ASSERT (sock);
    BSON_ASSERT (events);
 
-   pfd.fd = sd;
 #ifdef _WIN32
-   pfd.events = events;
-#else
-   pfd.events = events | POLLERR | POLLHUP;
-#endif
-   pfd.revents = 0;
+   FD_ZERO (&read_fds);
+   FD_ZERO (&write_fds);
+   FD_ZERO (&error_fds);
 
-   now = bson_get_monotonic_time();
+   if (events & POLLIN) {
+      FD_SET (sock->sd, &read_fds);
+   }
+
+   if (events & POLLOUT) {
+      FD_SET (sock->sd, &write_fds);
+   }
+
+   FD_SET (sock->sd, &error_fds);
+#else
+   pfd.fd = sock->sd;
+   pfd.events = events | POLLERR | POLLHUP;
+   pfd.revents = 0;
+#endif
+   now = bson_get_monotonic_time ();
 
    for (;;) {
       if (expire_at < 0) {
@@ -130,16 +176,27 @@ _mongoc_socket_wait (int      sd,           /* IN */
       } else if (expire_at == 0) {
          timeout = 0;
       } else {
-         timeout = (int)((expire_at - now) / 1000L);
+         timeout = (int) ((expire_at - now) / 1000L);
          if (timeout < 0) {
             timeout = 0;
          }
       }
 
 #ifdef _WIN32
-      ret = WSAPoll (&pfd, 1, timeout);
+      if (timeout == -1) {
+         /* not WSAPoll: daniel.haxx.se/blog/2012/10/10/wsapoll-is-broken */
+         ret = select (0 /*unused*/, &read_fds, &write_fds, &error_fds, NULL);
+      } else {
+         timeout_tv.tv_sec = timeout / 1000;
+         timeout_tv.tv_usec = (timeout % 1000) * 1000;
+         ret = select (
+            0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
+      }
       if (ret == SOCKET_ERROR) {
-         errno = WSAGetLastError();
+         _mongoc_socket_capture_errno (sock);
+         ret = -1;
+      } else if (FD_ISSET (sock->sd, &error_fds)) {
+         errno = WSAECONNRESET;
          ret = -1;
       }
 #else
@@ -147,30 +204,38 @@ _mongoc_socket_wait (int      sd,           /* IN */
 #endif
 
       if (ret > 0) {
-         /* Something happened, so return that */
+/* Something happened, so return that */
 #ifdef _WIN32
-         RETURN (0 != (pfd.revents & (events | POLLHUP | POLLERR)));
+         return (FD_ISSET (sock->sd, &read_fds)
+                 || FD_ISSET (sock->sd, &write_fds));
 #else
          RETURN (0 != (pfd.revents & events));
 #endif
       } else if (ret < 0) {
          /* poll itself failed */
 
-         TRACE("errno is: %d", errno);
-         if (MONGOC_ERRNO_IS_AGAIN(errno)) {
-            now = bson_get_monotonic_time();
+         TRACE ("errno is: %d", errno);
+         if (MONGOC_ERRNO_IS_AGAIN (errno)) {
+            now = bson_get_monotonic_time ();
 
             if (expire_at < now) {
+               _mongoc_socket_capture_errno (sock);
                RETURN (false);
             } else {
                continue;
             }
          } else {
             /* poll failed for some non-transient reason */
+            _mongoc_socket_capture_errno (sock);
             RETURN (false);
          }
       } else {
-         /* poll timed out */
+/* ret == 0, poll timed out */
+#ifdef _WIN32
+         sock->errno_ = timeout ? WSAETIMEDOUT : EAGAIN;
+#else
+         sock->errno_ = timeout ? ETIMEDOUT : EAGAIN;
+#endif
          RETURN (false);
       }
    }
@@ -180,20 +245,17 @@ _mongoc_socket_wait (int      sd,           /* IN */
 /*
  *--------------------------------------------------------------------------
  *
- * _mongoc_socket_poll --
+ * mongoc_socket_poll --
  *
  *       A multi-socket poll helper.
  *
- *       @events: in most cases should be POLLIN or POLLOUT.
- *
  *       @expire_at should be an absolute time at which to expire using
  *       the monotonic clock (bson_get_monotonic_time(), which is in
  *       microseconds). Or zero to not block at all. Or -1 to block
  *       forever.
  *
  * Returns:
- *       true if an event matched. otherwise false.
- *       a timeout will return false.
+ *       The number of sockets ready.
  *
  * Side effects:
  *       None.
@@ -202,12 +264,15 @@ _mongoc_socket_wait (int      sd,           /* IN */
  */
 
 ssize_t
-mongoc_socket_poll (mongoc_socket_poll_t *sds,          /* IN */
-                    size_t                nsds,         /* IN */
-                    int32_t               timeout)      /* IN */
+mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
+                    size_t nsds,               /* IN */
+                    int32_t timeout)           /* IN */
 {
 #ifdef _WIN32
-   WSAPOLLFD *pfds;
+   fd_set read_fds;
+   fd_set write_fds;
+   fd_set error_fds;
+   struct timeval timeout_tv;
 #else
    struct pollfd *pfds;
 #endif
@@ -219,46 +284,266 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds,          /* IN */
    BSON_ASSERT (sds);
 
 #ifdef _WIN32
-   pfds = (WSAPOLLFD *)bson_malloc(sizeof(*pfds) * nsds);
+   FD_ZERO (&read_fds);
+   FD_ZERO (&write_fds);
+   FD_ZERO (&error_fds);
+
+   for (i = 0; i < nsds; i++) {
+      if (sds[i].events & POLLIN) {
+         FD_SET (sds[i].socket->sd, &read_fds);
+      }
+
+      if (sds[i].events & POLLOUT) {
+         FD_SET (sds[i].socket->sd, &write_fds);
+      }
+
+      FD_SET (sds[i].socket->sd, &error_fds);
+   }
+
+   timeout_tv.tv_sec = timeout / 1000;
+   timeout_tv.tv_usec = (timeout % 1000) * 1000;
+
+   /* not WSAPoll: daniel.haxx.se/blog/2012/10/10/wsapoll-is-broken */
+   ret = select (0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
+   if (ret == SOCKET_ERROR) {
+      errno = WSAGetLastError ();
+      return -1;
+   }
+
+   for (i = 0; i < nsds; i++) {
+      if (FD_ISSET (sds[i].socket->sd, &read_fds)) {
+         sds[i].revents = POLLIN;
+      } else if (FD_ISSET (sds[i].socket->sd, &write_fds)) {
+         sds[i].revents = POLLOUT;
+      } else if (FD_ISSET (sds[i].socket->sd, &error_fds)) {
+         sds[i].revents = POLLHUP;
+      } else {
+         sds[i].revents = 0;
+      }
+   }
 #else
-   pfds = (struct pollfd *)bson_malloc(sizeof(*pfds) * nsds);
-#endif
+   pfds = (struct pollfd *) bson_malloc (sizeof (*pfds) * nsds);
 
    for (i = 0; i < nsds; i++) {
       pfds[i].fd = sds[i].socket->sd;
-#ifdef _WIN32
-      pfds[i].events = sds[i].events;
-#else
       pfds[i].events = sds[i].events | POLLERR | POLLHUP;
-#endif
       pfds[i].revents = 0;
    }
 
-#ifdef _WIN32
-   ret = WSAPoll (pfds, nsds, timeout);
-   if (ret == SOCKET_ERROR) {
-      MONGOC_WARNING ("WSAGetLastError(): %d", WSAGetLastError ());
-      ret = -1;
-   }
-#else
    ret = poll (pfds, nsds, timeout);
-#endif
-
    for (i = 0; i < nsds; i++) {
       sds[i].revents = pfds[i].revents;
    }
 
-   bson_free(pfds);
+   bson_free (pfds);
+#endif
 
    return ret;
 }
 
 
+/* https://jira.mongodb.org/browse/CDRIVER-2176 */
+#define MONGODB_KEEPALIVEINTVL 10
+#define MONGODB_KEEPIDLE 300
+#define MONGODB_KEEPALIVECNT 9
+
+#ifdef _WIN32
+static void
+_mongoc_socket_setkeepalive_windows (SOCKET sd)
+{
+   BOOL optval = 1;
+   struct tcp_keepalive keepalive;
+   DWORD lpcbBytesReturned = 0;
+   HKEY hKey;
+   DWORD type;
+   DWORD data;
+   DWORD data_size = sizeof data;
+   const char *reg_key =
+      "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters";
+   keepalive.onoff = true;
+   keepalive.keepalivetime = MONGODB_KEEPIDLE * 1000;
+   keepalive.keepaliveinterval = MONGODB_KEEPALIVEINTVL * 1000;
+   /*
+    * Windows hardcodes probes to 10:
+    * https://msdn.microsoft.com/en-us/library/windows/desktop/dd877220(v=vs.85).aspx
+    * "On Windows Vista and later, the number of keep-alive probes (data
+    * retransmissions) is set to 10 and cannot be changed."
+    *
+    * Note that win2k (and seeminly all versions thereafter) do not set the
+    * registry value by default so there is no way to derive the default value
+    * programmatically. It is however listed in the docs. A user can however
+    * change the default value by setting the registry values.
+    */
+
+   if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, reg_key, 0, KEY_QUERY_VALUE, &hKey) ==
+       ERROR_SUCCESS) {
+      /* https://technet.microsoft.com/en-us/library/cc957549.aspx */
+      DWORD default_keepalivetime = 7200000; /* 2 hours */
+      /* https://technet.microsoft.com/en-us/library/cc957548.aspx */
+      DWORD default_keepaliveinterval = 1000; /* 1 second */
+
+      if (RegQueryValueEx (
+             hKey, "KeepAliveTime", NULL, &type, (LPBYTE) &data, &data_size) ==
+          ERROR_SUCCESS) {
+         if (type == REG_DWORD && data < keepalive.keepalivetime) {
+            keepalive.keepalivetime = data;
+         }
+      } else if (default_keepalivetime < keepalive.keepalivetime) {
+         keepalive.keepalivetime = default_keepalivetime;
+      }
+
+      if (RegQueryValueEx (hKey,
+                           "KeepAliveInterval",
+                           NULL,
+                           &type,
+                           (LPBYTE) &data,
+                           &data_size) == ERROR_SUCCESS) {
+         if (type == REG_DWORD && data < keepalive.keepaliveinterval) {
+            keepalive.keepaliveinterval = data;
+         }
+      } else if (default_keepaliveinterval < keepalive.keepaliveinterval) {
+         keepalive.keepaliveinterval = default_keepaliveinterval;
+      }
+      RegCloseKey (hKey);
+   }
+   if (WSAIoctl (sd,
+                 SIO_KEEPALIVE_VALS,
+                 &keepalive,
+                 sizeof keepalive,
+                 NULL,
+                 0,
+                 &lpcbBytesReturned,
+                 NULL,
+                 NULL) == SOCKET_ERROR) {
+      TRACE ("Could not set keepalive values");
+   } else {
+      TRACE ("KeepAlive values updated");
+      TRACE ("KeepAliveTime: %d", keepalive.keepalivetime);
+      TRACE ("KeepAliveInterval: %d", keepalive.keepaliveinterval);
+   }
+}
+#else
+#ifdef MONGOC_TRACE
+static const char *
+_mongoc_socket_sockopt_value_to_name (int value)
+{
+   switch (value) {
+#ifdef TCP_KEEPIDLE
+   case TCP_KEEPIDLE:
+      return "TCP_KEEPIDLE";
+#endif
+#ifdef TCP_KEEPALIVE
+   case TCP_KEEPALIVE:
+      return "TCP_KEEPALIVE";
+#endif
+#ifdef TCP_KEEPINTVL
+   case TCP_KEEPINTVL:
+      return "TCP_KEEPINTVL";
+#endif
+#ifdef TCP_KEEPCNT
+   case TCP_KEEPCNT:
+      return "TCP_KEEPCNT";
+#endif
+   default:
+      MONGOC_WARNING ("Don't know what socketopt %d is", value);
+      return "Unknown option name";
+   }
+}
+#endif
+static void
+_mongoc_socket_set_sockopt_if_less (int sd, int name, int value)
+{
+   int optval = 1;
+   mongoc_socklen_t optlen;
+
+   optlen = sizeof optval;
+   if (getsockopt (sd, IPPROTO_TCP, name, (char *) &optval, &optlen)) {
+      TRACE ("Getting '%s' failed, errno: %d",
+             _mongoc_socket_sockopt_value_to_name (name),
+             errno);
+   } else {
+      TRACE ("'%s' is %d, target value is %d",
+             _mongoc_socket_sockopt_value_to_name (name),
+             optval,
+             value);
+      if (optval > value) {
+         optval = value;
+         if (setsockopt (
+                sd, IPPROTO_TCP, name, (char *) &optval, sizeof optval)) {
+            TRACE ("Setting '%s' failed, errno: %d",
+                   _mongoc_socket_sockopt_value_to_name (name),
+                   errno);
+         } else {
+            TRACE ("'%s' value changed to %d",
+                   _mongoc_socket_sockopt_value_to_name (name),
+                   optval);
+         }
+      }
+   }
+}
+
+static void
+_mongoc_socket_setkeepalive_nix (int sd)
+{
+#if defined(TCP_KEEPIDLE)
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPIDLE, MONGODB_KEEPIDLE);
+#elif defined(TCP_KEEPALIVE)
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPALIVE, MONGODB_KEEPIDLE);
+#else
+   TRACE ("%s", "Neither TCP_KEEPIDLE nor TCP_KEEPALIVE available");
+#endif
+
+#ifdef TCP_KEEPINTVL
+   _mongoc_socket_set_sockopt_if_less (
+      sd, TCP_KEEPINTVL, MONGODB_KEEPALIVEINTVL);
+#else
+   TRACE ("%s", "TCP_KEEPINTVL not available");
+#endif
+
+#ifdef TCP_KEEPCNT
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPCNT, MONGODB_KEEPALIVECNT);
+#else
+   TRACE ("%s", "TCP_KEEPCNT not available");
+#endif
+}
+
+#endif
+static void
+#ifdef _WIN32
+_mongoc_socket_setkeepalive (SOCKET sd) /* IN */
+#else
+_mongoc_socket_setkeepalive (int sd) /* IN */
+#endif
+{
+#ifdef SO_KEEPALIVE
+   int optval = 1;
+
+   ENTRY;
+#ifdef SO_KEEPALIVE
+   if (!setsockopt (
+          sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &optval, sizeof optval)) {
+      TRACE ("%s", "Setting SO_KEEPALIVE");
+#ifdef _WIN32
+      _mongoc_socket_setkeepalive_windows (sd);
+#else
+      _mongoc_socket_setkeepalive_nix (sd);
+#endif
+   } else {
+      TRACE ("%s", "Failed setting SO_KEEPALIVE");
+   }
+#else
+   TRACE ("%s", "SO_KEEPALIVE not available");
+#endif
+   EXIT;
+#endif
+}
+
+
 static bool
 #ifdef _WIN32
 _mongoc_socket_setnodelay (SOCKET sd) /* IN */
 #else
-_mongoc_socket_setnodelay (int sd)    /* IN */
+_mongoc_socket_setnodelay (int sd) /* IN */
 #endif
 {
 #ifdef _WIN32
@@ -271,12 +556,12 @@ _mongoc_socket_setnodelay (int sd)    /* IN */
    ENTRY;
 
    errno = 0;
-   ret = setsockopt (sd, IPPROTO_TCP, TCP_NODELAY,
-                     (char *)&optval, sizeof optval);
+   ret = setsockopt (
+      sd, IPPROTO_TCP, TCP_NODELAY, (char *) &optval, sizeof optval);
 
 #ifdef _WIN32
    if (ret == SOCKET_ERROR) {
-      MONGOC_WARNING ("WSAGetLastError(): %d", (int)WSAGetLastError ());
+      MONGOC_WARNING ("WSAGetLastError(): %d", (int) WSAGetLastError ());
    }
 #endif
 
@@ -304,39 +589,11 @@ int
 mongoc_socket_errno (mongoc_socket_t *sock) /* IN */
 {
    BSON_ASSERT (sock);
-   TRACE("Current errno: %d", sock->errno_);
+   TRACE ("Current errno: %d", sock->errno_);
    return sock->errno_;
 }
 
 
-/*
- *--------------------------------------------------------------------------
- *
- * _mongoc_socket_capture_errno --
- *
- *       Save the errno state for contextual use.
- *
- * Returns:
- *       None.
- *
- * Side effects:
- *       None.
- *
- *--------------------------------------------------------------------------
- */
-
-static void
-_mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
-{
-#ifdef _WIN32
-   errno = sock->errno_ = WSAGetLastError ();
-#else
-   sock->errno_ = errno;
-#endif
-   TRACE("setting errno: %d", sock->errno_);
-}
-
-
 /*
  *--------------------------------------------------------------------------
  *
@@ -357,7 +614,7 @@ _mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
 static bool
 _mongoc_socket_errno_is_again (mongoc_socket_t *sock) /* IN */
 {
-   TRACE("errno is: %d", sock->errno_);
+   TRACE ("errno is: %d", sock->errno_);
    return MONGOC_ERRNO_IS_AGAIN (sock->errno_);
 }
 
@@ -381,8 +638,8 @@ _mongoc_socket_errno_is_again (mongoc_socket_t *sock) /* IN */
  */
 
 mongoc_socket_t *
-mongoc_socket_accept (mongoc_socket_t *sock,      /* IN */
-                      int64_t          expire_at) /* IN */
+mongoc_socket_accept (mongoc_socket_t *sock, /* IN */
+                      int64_t expire_at)     /* IN */
 {
    return mongoc_socket_accept_ex (sock, expire_at, NULL);
 }
@@ -406,13 +663,13 @@ mongoc_socket_accept (mongoc_socket_t *sock,      /* IN */
  */
 
 mongoc_socket_t *
-mongoc_socket_accept_ex (mongoc_socket_t *sock,      /* IN */
-                         int64_t          expire_at, /* IN */
-                         uint16_t *port)             /* OUT */
+mongoc_socket_accept_ex (mongoc_socket_t *sock, /* IN */
+                         int64_t expire_at,     /* IN */
+                         uint16_t *port)        /* OUT */
 {
    mongoc_socket_t *client;
-   struct sockaddr_in addr = { 0 };
-   socklen_t addrlen = sizeof addr;
+   struct sockaddr_in addr = {0};
+   mongoc_socklen_t addrlen = sizeof addr;
    bool try_again = false;
    bool failed = false;
 #ifdef _WIN32
@@ -427,7 +684,7 @@ mongoc_socket_accept_ex (mongoc_socket_t *sock,      /* IN */
 
 again:
    errno = 0;
-   sd = accept (sock->sd, (struct sockaddr *) &addr, &addrlen);
+   sd = accept (sock->sd, (mongoc_sockaddr_t *) &addr, &addrlen);
 
    _mongoc_socket_capture_errno (sock);
 #ifdef _WIN32
@@ -438,7 +695,7 @@ again:
    try_again = (failed && _mongoc_socket_errno_is_again (sock));
 
    if (failed && try_again) {
-      if (_mongoc_socket_wait (sock->sd, POLLIN, expire_at)) {
+      if (_mongoc_socket_wait (sock, POLLIN, expire_at)) {
          GOTO (again);
       }
       RETURN (NULL);
@@ -453,7 +710,7 @@ again:
       RETURN (NULL);
    }
 
-   client = (mongoc_socket_t *)bson_malloc0 (sizeof *client);
+   client = (mongoc_socket_t *) bson_malloc0 (sizeof *client);
    client->sd = sd;
 
    if (port) {
@@ -485,9 +742,9 @@ again:
  */
 
 int
-mongoc_socket_bind (mongoc_socket_t       *sock,    /* IN */
-                    const struct sockaddr *addr,    /* IN */
-                    socklen_t              addrlen) /* IN */
+mongoc_socket_bind (mongoc_socket_t *sock,       /* IN */
+                    const struct sockaddr *addr, /* IN */
+                    mongoc_socklen_t addrlen)    /* IN */
 {
    int ret;
 
@@ -505,55 +762,45 @@ mongoc_socket_bind (mongoc_socket_t       *sock,    /* IN */
 }
 
 
-/*
- *--------------------------------------------------------------------------
- *
- * mongoc_socket_close --
- *
- *       Closes the underlying socket.
- *
- *       In general, you probably don't want to handle the result from
- *       this. That could cause race conditions in the case of preemtion
- *       during system call (EINTR).
- *
- * Returns:
- *       0 on success, -1 on failure.
- *
- * Side effects:
- *       None.
- *
- *--------------------------------------------------------------------------
- */
-
 int
 mongoc_socket_close (mongoc_socket_t *sock) /* IN */
 {
+   bool owned;
+
    ENTRY;
 
    BSON_ASSERT (sock);
 
+   owned = (sock->pid == (int) getpid ());
+
 #ifdef _WIN32
    if (sock->sd != INVALID_SOCKET) {
-      shutdown (sock->sd, SD_BOTH);
+      if (owned) {
+         shutdown (sock->sd, SD_BOTH);
+      }
+
       if (0 == closesocket (sock->sd)) {
          sock->sd = INVALID_SOCKET;
       } else {
          _mongoc_socket_capture_errno (sock);
-         RETURN(-1);
+         RETURN (-1);
       }
    }
-   RETURN(0);
+   RETURN (0);
 #else
    if (sock->sd != -1) {
-      shutdown (sock->sd, SHUT_RDWR);
+      if (owned) {
+         shutdown (sock->sd, SHUT_RDWR);
+      }
+
       if (0 == close (sock->sd)) {
          sock->sd = -1;
       } else {
          _mongoc_socket_capture_errno (sock);
-         RETURN(-1);
+         RETURN (-1);
       }
    }
-   RETURN(0);
+   RETURN (0);
 #endif
 }
 
@@ -576,16 +823,17 @@ mongoc_socket_close (mongoc_socket_t *sock) /* IN */
  */
 
 int
-mongoc_socket_connect (mongoc_socket_t       *sock,      /* IN */
-                       const struct sockaddr *addr,      /* IN */
-                       socklen_t              addrlen,   /* IN */
-                       int64_t                expire_at) /* IN */
+mongoc_socket_connect (mongoc_socket_t *sock,       /* IN */
+                       const struct sockaddr *addr, /* IN */
+                       mongoc_socklen_t addrlen,    /* IN */
+                       int64_t expire_at)           /* IN */
 {
    bool try_again = false;
    bool failed = false;
    int ret;
    int optval;
-   socklen_t optlen = sizeof optval;
+   /* getsockopt parameter types vary, we check in CheckCompiler.m4 */
+   mongoc_socklen_t optlen = (mongoc_socklen_t) sizeof optval;
 
    ENTRY;
 
@@ -607,10 +855,10 @@ mongoc_socket_connect (mongoc_socket_t       *sock,      /* IN */
    }
 
    if (failed && try_again) {
-      if (_mongoc_socket_wait (sock->sd, POLLOUT, expire_at)) {
+      if (_mongoc_socket_wait (sock, POLLOUT, expire_at)) {
          optval = -1;
-         ret = getsockopt (sock->sd, SOL_SOCKET, SO_ERROR,
-                           (char *)&optval, &optlen);
+         ret = getsockopt (
+            sock->sd, SOL_SOCKET, SO_ERROR, (char *) &optval, &optlen);
          if ((ret == 0) && (optval == 0)) {
             RETURN (0);
          } else {
@@ -672,8 +920,8 @@ mongoc_socket_destroy (mongoc_socket_t *sock) /* IN */
  */
 
 int
-mongoc_socket_listen (mongoc_socket_t *sock,    /* IN */
-                      unsigned int     backlog) /* IN */
+mongoc_socket_listen (mongoc_socket_t *sock, /* IN */
+                      unsigned int backlog)  /* IN */
 {
    int ret;
 
@@ -698,9 +946,9 @@ mongoc_socket_listen (mongoc_socket_t *sock,    /* IN */
  *
  * mongoc_socket_new --
  *
- *       Create a new socket.
+ *       Create a new socket and store the current process id on it.
  *
- *       Free the result mongoc_socket_destroy().
+ *       Free the result with mongoc_socket_destroy().
  *
  * Returns:
  *       A newly allocated socket.
@@ -740,13 +988,17 @@ mongoc_socket_new (int domain,   /* IN */
       GOTO (fail);
    }
 
-   if (domain != AF_UNIX && !_mongoc_socket_setnodelay (sd)) {
-      MONGOC_WARNING ("Failed to enable TCP_NODELAY.");
+   if (domain != AF_UNIX) {
+      if (!_mongoc_socket_setnodelay (sd)) {
+         MONGOC_WARNING ("Failed to enable TCP_NODELAY.");
+      }
+      _mongoc_socket_setkeepalive (sd);
    }
 
-   sock = (mongoc_socket_t *)bson_malloc0 (sizeof *sock);
+   sock = (mongoc_socket_t *) bson_malloc0 (sizeof *sock);
    sock->sd = sd;
    sock->domain = domain;
+   sock->pid = (int) getpid ();
 
    RETURN (sock);
 
@@ -785,11 +1037,11 @@ fail:
  */
 
 ssize_t
-mongoc_socket_recv (mongoc_socket_t *sock,      /* IN */
-                    void            *buf,       /* OUT */
-                    size_t           buflen,    /* IN */
-                    int              flags,     /* IN */
-                    int64_t          expire_at) /* IN */
+mongoc_socket_recv (mongoc_socket_t *sock, /* IN */
+                    void *buf,             /* OUT */
+                    size_t buflen,         /* IN */
+                    int flags,             /* IN */
+                    int64_t expire_at)     /* IN */
 {
    ssize_t ret = 0;
    bool failed = false;
@@ -803,7 +1055,7 @@ mongoc_socket_recv (mongoc_socket_t *sock,      /* IN */
 again:
    sock->errno_ = 0;
 #ifdef _WIN32
-   ret = recv (sock->sd, (char *)buf, (int)buflen, flags);
+   ret = recv (sock->sd, (char *) buf, (int) buflen, flags);
    failed = (ret == SOCKET_ERROR);
 #else
    ret = recv (sock->sd, buf, buflen, flags);
@@ -812,7 +1064,7 @@ again:
    if (failed) {
       _mongoc_socket_capture_errno (sock);
       if (_mongoc_socket_errno_is_again (sock) &&
-          _mongoc_socket_wait (sock->sd, POLLIN, expire_at)) {
+          _mongoc_socket_wait (sock, POLLIN, expire_at)) {
          GOTO (again);
       }
    }
@@ -844,11 +1096,11 @@ again:
  */
 
 int
-mongoc_socket_setsockopt (mongoc_socket_t *sock,    /* IN */
-                          int              level,   /* IN */
-                          int              optname, /* IN */
-                          const void      *optval,  /* IN */
-                          socklen_t        optlen)  /* IN */
+mongoc_socket_setsockopt (mongoc_socket_t *sock,   /* IN */
+                          int level,               /* IN */
+                          int optname,             /* IN */
+                          const void *optval,      /* IN */
+                          mongoc_socklen_t optlen) /* IN */
 {
    int ret;
 
@@ -885,10 +1137,10 @@ mongoc_socket_setsockopt (mongoc_socket_t *sock,    /* IN */
  */
 
 ssize_t
-mongoc_socket_send (mongoc_socket_t *sock,      /* IN */
-                    const void      *buf,       /* IN */
-                    size_t           buflen,    /* IN */
-                    int64_t          expire_at) /* IN */
+mongoc_socket_send (mongoc_socket_t *sock, /* IN */
+                    const void *buf,       /* IN */
+                    size_t buflen,         /* IN */
+                    int64_t expire_at)     /* IN */
 {
    mongoc_iovec_t iov;
 
@@ -896,7 +1148,7 @@ mongoc_socket_send (mongoc_socket_t *sock,      /* IN */
    BSON_ASSERT (buf);
    BSON_ASSERT (buflen);
 
-   iov.iov_base = (void *)buf;
+   iov.iov_base = (void *) buf;
    iov.iov_len = buflen;
 
    return mongoc_socket_sendv (sock, &iov, 1, expire_at);
@@ -923,9 +1175,9 @@ mongoc_socket_send (mongoc_socket_t *sock,      /* IN */
  */
 
 static ssize_t
-_mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
-                               mongoc_iovec_t  *iov,    /* IN */
-                               size_t           iovcnt) /* IN */
+_mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
+                               mongoc_iovec_t *iov,   /* IN */
+                               size_t iovcnt)         /* IN */
 {
    ssize_t ret = 0;
    size_t i;
@@ -938,7 +1190,7 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
    BSON_ASSERT (iovcnt);
 
    for (i = 0; i < iovcnt; i++) {
-      wrote = send (sock->sd, iov [i].iov_base, iov [i].iov_len, 0);
+      wrote = send (sock->sd, iov[i].iov_base, iov[i].iov_len, 0);
 #ifdef _WIN32
       if (wrote == SOCKET_ERROR) {
 #else
@@ -954,7 +1206,7 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
 
       ret += wrote;
 
-      if (wrote != iov [i].iov_len) {
+      if (wrote != iov[i].iov_len) {
          RETURN (ret);
       }
    }
@@ -983,9 +1235,9 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock,   /* IN */
  */
 
 static ssize_t
-_mongoc_socket_try_sendv (mongoc_socket_t *sock,   /* IN */
-                          mongoc_iovec_t  *iov,    /* IN */
-                          size_t           iovcnt) /* IN */
+_mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
+                          mongoc_iovec_t *iov,   /* IN */
+                          size_t iovcnt)         /* IN */
 {
 #ifdef _WIN32
    DWORD dwNumberofBytesSent = 0;
@@ -1004,20 +1256,24 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock,   /* IN */
    DUMP_IOVEC (sendbuf, iov, iovcnt);
 
 #ifdef _WIN32
-   ret = WSASend (sock->sd, (LPWSABUF)iov, iovcnt, &dwNumberofBytesSent,
-                  0, NULL, NULL);
-   TRACE("WSASend sent: %ld (out of: %ld), ret: %d", dwNumberofBytesSent, iov->iov_len, ret);
+   ret = WSASend (
+      sock->sd, (LPWSABUF) iov, iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
+   TRACE ("WSASend sent: %ld (out of: %ld), ret: %d",
+          dwNumberofBytesSent,
+          iov->iov_len,
+          ret);
 #else
    memset (&msg, 0, sizeof msg);
    msg.msg_iov = iov;
    msg.msg_iovlen = (int) iovcnt;
-   ret = sendmsg (sock->sd, &msg,
-# ifdef MSG_NOSIGNAL
+   ret = sendmsg (sock->sd,
+                  &msg,
+#ifdef MSG_NOSIGNAL
                   MSG_NOSIGNAL);
-# else
+#else
                   0);
-# endif
-   TRACE("Send %ld out of %ld bytes", ret, iov->iov_len);
+#endif
+   TRACE ("Send %ld out of %ld bytes", ret, iov->iov_len);
 #endif
 
 
@@ -1028,17 +1284,17 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock,   /* IN */
 #endif
       _mongoc_socket_capture_errno (sock);
 
-      /*
-       * Check to see if we have sent an iovec too large for sendmsg to
-       * complete. If so, we need to fallback to the slow path of multiple
-       * send() commands.
-       */
+/*
+ * Check to see if we have sent an iovec too large for sendmsg to
+ * complete. If so, we need to fallback to the slow path of multiple
+ * send() commands.
+ */
 #ifdef _WIN32
       if (mongoc_socket_errno (sock) == WSAEMSGSIZE) {
 #else
       if (mongoc_socket_errno (sock) == EMSGSIZE) {
 #endif
-         RETURN(_mongoc_socket_try_sendv_slow (sock, iov, iovcnt));
+         RETURN (_mongoc_socket_try_sendv_slow (sock, iov, iovcnt));
       }
 
       RETURN (-1);
@@ -1076,10 +1332,10 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock,   /* IN */
  */
 
 ssize_t
-mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
-                     mongoc_iovec_t   *in_iov,    /* IN */
-                     size_t            iovcnt,    /* IN */
-                     int64_t           expire_at) /* IN */
+mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
+                     mongoc_iovec_t *in_iov, /* IN */
+                     size_t iovcnt,          /* IN */
+                     int64_t expire_at)      /* IN */
 {
    ssize_t ret = 0;
    ssize_t sent;
@@ -1092,12 +1348,13 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
    BSON_ASSERT (in_iov);
    BSON_ASSERT (iovcnt);
 
-   iov = bson_malloc(sizeof(*iov) * iovcnt);
-   memcpy(iov, in_iov, sizeof(*iov) * iovcnt);
+   iov = bson_malloc (sizeof (*iov) * iovcnt);
+   memcpy (iov, in_iov, sizeof (*iov) * iovcnt);
 
    for (;;) {
-      sent = _mongoc_socket_try_sendv (sock, &iov [cur], iovcnt - cur);
-      TRACE("Sent %ld (of %ld) out of iovcnt=%ld", sent, iov[cur].iov_len, iovcnt);
+      sent = _mongoc_socket_try_sendv (sock, &iov[cur], iovcnt - cur);
+      TRACE (
+         "Sent %ld (of %ld) out of iovcnt=%ld", sent, iov[cur].iov_len, iovcnt);
 
       /*
        * If we failed with anything other than EAGAIN or EWOULDBLOCK,
@@ -1107,7 +1364,7 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
       if (sent == -1) {
          if (!_mongoc_socket_errno_is_again (sock)) {
             ret = -1;
-            GOTO(CLEANUP);
+            GOTO (CLEANUP);
          }
       }
 
@@ -1121,9 +1378,11 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
          /*
           * Subtract the sent amount from what we still need to send.
           */
-         while ((cur < iovcnt) && (sent >= (ssize_t)iov [cur].iov_len)) {
-            TRACE("still got bytes left: sent -= iov_len: %ld -= %ld", sent, iov[cur].iov_len);
-            sent -= iov [cur++].iov_len;
+         while ((cur < iovcnt) && (sent >= (ssize_t) iov[cur].iov_len)) {
+            TRACE ("still got bytes left: sent -= iov_len: %ld -= %ld",
+                   sent,
+                   iov[cur].iov_len);
+            sent -= iov[cur++].iov_len;
          }
 
          /*
@@ -1131,7 +1390,7 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
           * sending data over the socket.
           */
          if (cur == iovcnt) {
-            TRACE("%s", "Finished the iovecs");
+            TRACE ("%s", "Finished the iovecs");
             break;
          }
 
@@ -1139,37 +1398,38 @@ mongoc_socket_sendv (mongoc_socket_t  *sock,      /* IN */
           * Increment the current iovec buffer to its proper offset and adjust
           * the number of bytes to write.
           */
-         TRACE("Seeked io_base+%ld", sent);
-         TRACE("Subtracting iov_len -= sent; %ld -= %ld", iov[cur].iov_len, sent);
-         iov [cur].iov_base = ((char *)iov [cur].iov_base) + sent;
-         iov [cur].iov_len -= sent;
-         TRACE("iov_len remaining %ld", iov[cur].iov_len);
+         TRACE ("Seeked io_base+%ld", sent);
+         TRACE (
+            "Subtracting iov_len -= sent; %ld -= %ld", iov[cur].iov_len, sent);
+         iov[cur].iov_base = ((char *) iov[cur].iov_base) + sent;
+         iov[cur].iov_len -= sent;
+         TRACE ("iov_len remaining %ld", iov[cur].iov_len);
 
          BSON_ASSERT (iovcnt - cur);
-         BSON_ASSERT (iov [cur].iov_len);
+         BSON_ASSERT (iov[cur].iov_len);
       } else if (OPERATION_EXPIRED (expire_at)) {
-         GOTO(CLEANUP);
+         GOTO (CLEANUP);
       }
 
       /*
        * Block on poll() until our desired condition is met.
        */
-      if (!_mongoc_socket_wait (sock->sd, POLLOUT, expire_at)) {
-         GOTO(CLEANUP);
+      if (!_mongoc_socket_wait (sock, POLLOUT, expire_at)) {
+         GOTO (CLEANUP);
       }
    }
 
 CLEANUP:
-   bson_free(iov);
+   bson_free (iov);
 
    RETURN (ret);
 }
 
 
 int
-mongoc_socket_getsockname (mongoc_socket_t *sock,    /* IN */
-                           struct sockaddr *addr,    /* OUT */
-                           socklen_t       *addrlen) /* INOUT */
+mongoc_socket_getsockname (mongoc_socket_t *sock,     /* IN */
+                           struct sockaddr *addr,     /* OUT */
+                           mongoc_socklen_t *addrlen) /* INOUT */
 {
    int ret;
 
@@ -1177,7 +1437,7 @@ mongoc_socket_getsockname (mongoc_socket_t *sock,    /* IN */
 
    BSON_ASSERT (sock);
 
-   ret = getsockname (sock->sd, addr, addrlen);
+   ret = getsockname (sock->sd, (mongoc_sockaddr_t *) addr, addrlen);
 
    _mongoc_socket_capture_errno (sock);
 
@@ -1188,10 +1448,11 @@ mongoc_socket_getsockname (mongoc_socket_t *sock,    /* IN */
 char *
 mongoc_socket_getnameinfo (mongoc_socket_t *sock) /* IN */
 {
-   struct sockaddr addr;
-   socklen_t len = sizeof addr;
+   /* getpeername parameter types vary, we check in CheckCompiler.m4 */
+   mongoc_sockaddr_t addr;
+   mongoc_socklen_t len = (mongoc_socklen_t) sizeof addr;
    char *ret;
-   char host [BSON_HOST_NAME_MAX + 1];
+   char host[BSON_HOST_NAME_MAX + 1];
 
    ENTRY;
 
@@ -1211,10 +1472,10 @@ bool
 mongoc_socket_check_closed (mongoc_socket_t *sock) /* IN */
 {
    bool closed = false;
-   char buf [1];
+   char buf[1];
    ssize_t r;
 
-   if (_mongoc_socket_wait (sock->sd, POLLIN, 0)) {
+   if (_mongoc_socket_wait (sock, POLLIN, 0)) {
       sock->errno_ = 0;
 
       r = recv (sock->sd, buf, 1, MSG_PEEK);
@@ -1250,21 +1511,21 @@ mongoc_socket_check_closed (mongoc_socket_t *sock) /* IN */
  */
 
 void
-mongoc_socket_inet_ntop (struct addrinfo *rp,        /* IN */
-                         char            *buf,       /* INOUT */
-                         size_t           buflen)    /* IN */
+mongoc_socket_inet_ntop (struct addrinfo *rp, /* IN */
+                         char *buf,           /* INOUT */
+                         size_t buflen)       /* IN */
 {
    void *ptr;
    char tmp[256];
 
    switch (rp->ai_family) {
    case AF_INET:
-      ptr = &((struct sockaddr_in *)rp->ai_addr)->sin_addr;
+      ptr = &((struct sockaddr_in *) rp->ai_addr)->sin_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
       bson_snprintf (buf, buflen, "ipv4 %s", tmp);
       break;
    case AF_INET6:
-      ptr = &((struct sockaddr_in6 *)rp->ai_addr)->sin6_addr;
+      ptr = &((struct sockaddr_in6 *) rp->ai_addr)->sin6_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
       bson_snprintf (buf, buflen, "ipv6 %s", tmp);
       break;
