@@ -17,7 +17,7 @@
 #ifndef MONGOC_STREAM_PRIVATE_H
 #define MONGOC_STREAM_PRIVATE_H
 
-#if !defined (MONGOC_I_AM_A_DRIVER) && !defined (MONGOC_COMPILATION)
+#if !defined(MONGOC_COMPILATION)
 #error "Only <mongoc.h> can be included directly."
 #endif
 
@@ -28,22 +28,21 @@
 BSON_BEGIN_DECLS
 
 
-#define MONGOC_STREAM_SOCKET   1
-#define MONGOC_STREAM_FILE     2
+#define MONGOC_STREAM_SOCKET 1
+#define MONGOC_STREAM_FILE 2
 #define MONGOC_STREAM_BUFFERED 3
-#define MONGOC_STREAM_GRIDFS   4
-#define MONGOC_STREAM_TLS      5
+#define MONGOC_STREAM_GRIDFS 4
+#define MONGOC_STREAM_TLS 5
 
 bool
-mongoc_stream_wait (mongoc_stream_t *stream,
-                    int64_t expire_at);
+mongoc_stream_wait (mongoc_stream_t *stream, int64_t expire_at);
 
 bool
 _mongoc_stream_writev_full (mongoc_stream_t *stream,
-                            mongoc_iovec_t  *iov,
-                            size_t           iovcnt,
-                            int32_t          timeout_msec,
-                            bson_error_t    *error);
+                            mongoc_iovec_t *iov,
+                            size_t iovcnt,
+                            int32_t timeout_msec,
+                            bson_error_t *error);
 
 
 BSON_END_DECLS
