@@ -18,15 +18,15 @@
 #include "mongoc-stream-private.h"
 #include "mongoc-stream-socket.h"
 #include "mongoc-trace-private.h"
-
+#include "mongoc-socket-private.h"
+#include "mongoc-errno-private.h"
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "stream"
 
 
-struct _mongoc_stream_socket_t
-{
-   mongoc_stream_t  vtable;
+struct _mongoc_stream_socket_t {
+   mongoc_stream_t vtable;
    mongoc_socket_t *sock;
 };
 
@@ -39,7 +39,7 @@ get_expiration (int32_t timeout_msec)
    } else if (timeout_msec == 0) {
       return 0;
    } else {
-      return (bson_get_monotonic_time () + ((int64_t)timeout_msec * 1000L));
+      return (bson_get_monotonic_time () + ((int64_t) timeout_msec * 1000L));
    }
 }
 
@@ -47,7 +47,7 @@ get_expiration (int32_t timeout_msec)
 static int
 _mongoc_stream_socket_close (mongoc_stream_t *stream)
 {
-   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *)stream;
+   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *) stream;
    int ret;
 
    ENTRY;
@@ -66,7 +66,7 @@ _mongoc_stream_socket_close (mongoc_stream_t *stream)
 static void
 _mongoc_stream_socket_destroy (mongoc_stream_t *stream)
 {
-   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *)stream;
+   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *) stream;
 
    ENTRY;
 
@@ -96,12 +96,12 @@ _mongoc_stream_socket_failed (mongoc_stream_t *stream)
 
 static int
 _mongoc_stream_socket_setsockopt (mongoc_stream_t *stream,
-                                  int              level,
-                                  int              optname,
-                                  void            *optval,
-                                  socklen_t        optlen)
+                                  int level,
+                                  int optname,
+                                  void *optval,
+                                  mongoc_socklen_t optlen)
 {
-   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *)stream;
+   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *) stream;
    int ret;
 
    ENTRY;
@@ -125,12 +125,12 @@ _mongoc_stream_socket_flush (mongoc_stream_t *stream)
 
 static ssize_t
 _mongoc_stream_socket_readv (mongoc_stream_t *stream,
-                             mongoc_iovec_t  *iov,
-                             size_t           iovcnt,
-                             size_t           min_bytes,
-                             int32_t          timeout_msec)
+                             mongoc_iovec_t *iov,
+                             size_t iovcnt,
+                             size_t min_bytes,
+                             int32_t timeout_msec)
 {
-   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *)stream;
+   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *) stream;
    int64_t expire_at;
    ssize_t ret = 0;
    ssize_t nread;
@@ -150,14 +150,11 @@ _mongoc_stream_socket_readv (mongoc_stream_t *stream,
     */
 
    for (;;) {
-      nread = mongoc_socket_recv (ss->sock,
-                                  iov [cur].iov_base,
-                                  iov [cur].iov_len,
-                                  0,
-                                  expire_at);
+      nread = mongoc_socket_recv (
+         ss->sock, iov[cur].iov_base, iov[cur].iov_len, 0, expire_at);
 
       if (nread <= 0) {
-         if (ret >= (ssize_t)min_bytes) {
+         if (ret >= (ssize_t) min_bytes) {
             RETURN (ret);
          }
          errno = mongoc_socket_errno (ss->sock);
@@ -166,23 +163,23 @@ _mongoc_stream_socket_readv (mongoc_stream_t *stream,
 
       ret += nread;
 
-      while ((cur < iovcnt) && (nread >= (ssize_t)iov [cur].iov_len)) {
-         nread -= iov [cur++].iov_len;
+      while ((cur < iovcnt) && (nread >= (ssize_t) iov[cur].iov_len)) {
+         nread -= iov[cur++].iov_len;
       }
 
       if (cur == iovcnt) {
          break;
       }
 
-      if (ret >= (ssize_t)min_bytes) {
+      if (ret >= (ssize_t) min_bytes) {
          RETURN (ret);
       }
 
-      iov [cur].iov_base = ((char *)iov [cur].iov_base) + nread;
-      iov [cur].iov_len -= nread;
+      iov[cur].iov_base = ((char *) iov[cur].iov_base) + nread;
+      iov[cur].iov_len -= nread;
 
       BSON_ASSERT (iovcnt - cur);
-      BSON_ASSERT (iov [cur].iov_len);
+      BSON_ASSERT (iov[cur].iov_len);
    }
 
    RETURN (ret);
@@ -191,11 +188,11 @@ _mongoc_stream_socket_readv (mongoc_stream_t *stream,
 
 static ssize_t
 _mongoc_stream_socket_writev (mongoc_stream_t *stream,
-                              mongoc_iovec_t  *iov,
-                              size_t           iovcnt,
-                              int32_t          timeout_msec)
+                              mongoc_iovec_t *iov,
+                              size_t iovcnt,
+                              int32_t timeout_msec)
 {
-   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *)stream;
+   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *) stream;
    int64_t expire_at;
    ssize_t ret;
 
@@ -214,8 +211,8 @@ _mongoc_stream_socket_writev (mongoc_stream_t *stream,
 
 static ssize_t
 _mongoc_stream_socket_poll (mongoc_stream_poll_t *streams,
-                            size_t                nstreams,
-                            int32_t               timeout_msec)
+                            size_t nstreams,
+                            int32_t timeout_msec)
 
 {
    int i;
@@ -225,12 +222,12 @@ _mongoc_stream_socket_poll (mongoc_stream_poll_t *streams,
 
    ENTRY;
 
-   sds = (mongoc_socket_poll_t *)bson_malloc(sizeof(*sds) * nstreams);
+   sds = (mongoc_socket_poll_t *) bson_malloc (sizeof (*sds) * nstreams);
 
    for (i = 0; i < nstreams; i++) {
-      ss = (mongoc_stream_socket_t *)streams[i].stream;
+      ss = (mongoc_stream_socket_t *) streams[i].stream;
 
-      if (! ss->sock) {
+      if (!ss->sock) {
          goto CLEANUP;
       }
 
@@ -238,7 +235,7 @@ _mongoc_stream_socket_poll (mongoc_stream_poll_t *streams,
       sds[i].events = streams[i].events;
    }
 
-   ret = mongoc_socket_poll(sds, nstreams, timeout_msec);
+   ret = mongoc_socket_poll (sds, nstreams, timeout_msec);
 
    if (ret > 0) {
       for (i = 0; i < nstreams; i++) {
@@ -247,7 +244,7 @@ _mongoc_stream_socket_poll (mongoc_stream_poll_t *streams,
    }
 
 CLEANUP:
-   bson_free(sds);
+   bson_free (sds);
 
    RETURN (ret);
 }
@@ -265,7 +262,7 @@ mongoc_stream_socket_get_socket (mongoc_stream_socket_t *stream) /* IN */
 static bool
 _mongoc_stream_socket_check_closed (mongoc_stream_t *stream) /* IN */
 {
-   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *)stream;
+   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *) stream;
 
    ENTRY;
 
@@ -279,6 +276,20 @@ _mongoc_stream_socket_check_closed (mongoc_stream_t *stream) /* IN */
 }
 
 
+static bool
+_mongoc_stream_socket_timed_out (mongoc_stream_t *stream) /* IN */
+{
+   mongoc_stream_socket_t *ss = (mongoc_stream_socket_t *) stream;
+
+   ENTRY;
+
+   BSON_ASSERT (ss);
+   BSON_ASSERT (ss->sock);
+
+   RETURN (MONGOC_ERRNO_IS_TIMEDOUT (ss->sock->errno_));
+}
+
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -303,7 +314,7 @@ mongoc_stream_socket_new (mongoc_socket_t *sock) /* IN */
 
    BSON_ASSERT (sock);
 
-   stream = (mongoc_stream_socket_t *)bson_malloc0 (sizeof *stream);
+   stream = (mongoc_stream_socket_t *) bson_malloc0 (sizeof *stream);
    stream->vtable.type = MONGOC_STREAM_SOCKET;
    stream->vtable.close = _mongoc_stream_socket_close;
    stream->vtable.destroy = _mongoc_stream_socket_destroy;
@@ -313,8 +324,9 @@ mongoc_stream_socket_new (mongoc_socket_t *sock) /* IN */
    stream->vtable.writev = _mongoc_stream_socket_writev;
    stream->vtable.setsockopt = _mongoc_stream_socket_setsockopt;
    stream->vtable.check_closed = _mongoc_stream_socket_check_closed;
+   stream->vtable.timed_out = _mongoc_stream_socket_timed_out;
    stream->vtable.poll = _mongoc_stream_socket_poll;
    stream->sock = sock;
 
-   return (mongoc_stream_t *)stream;
+   return (mongoc_stream_t *) stream;
 }
