@@ -24,6 +24,7 @@
 #include "mongoc-topology-private.h"
 #include "mongoc-topology-description-apm-private.h"
 #include "mongoc-client-private.h"
+#include "mongoc-uri-private.h"
 #include "mongoc-util-private.h"
 
 #include "utlist.h"
@@ -35,26 +36,27 @@ static void
 _mongoc_topology_request_scan (mongoc_topology_t *topology);
 
 static bool
-_mongoc_topology_reconcile_add_nodes (void *item,
-                                      void *ctx)
+_mongoc_topology_reconcile_add_nodes (void *item, void *ctx)
 {
    mongoc_server_description_t *sd = item;
-   mongoc_topology_t *topology = (mongoc_topology_t *)ctx;
+   mongoc_topology_t *topology = (mongoc_topology_t *) ctx;
    mongoc_topology_scanner_t *scanner = topology->scanner;
 
    /* quickly search by id, then check if a node for this host was retired in
     * this scan. */
-   if (! mongoc_topology_scanner_get_node (scanner, sd->id) &&
-       ! mongoc_topology_scanner_has_node_for_host (scanner, &sd->host)) {
-      mongoc_topology_scanner_add_and_scan (scanner, &sd->host, sd->id,
-                                            topology->connect_timeout_msec);
+   if (!mongoc_topology_scanner_get_node (scanner, sd->id) &&
+       !mongoc_topology_scanner_has_node_for_host (scanner, &sd->host)) {
+      mongoc_topology_scanner_add (scanner, &sd->host, sd->id);
+      mongoc_topology_scanner_scan (
+         scanner, sd->id, topology->connect_timeout_msec);
    }
 
    return true;
 }
 
 void
-mongoc_topology_reconcile (mongoc_topology_t *topology) {
+mongoc_topology_reconcile (mongoc_topology_t *topology)
+{
    mongoc_topology_scanner_node_t *ele, *tmp;
    mongoc_topology_description_t *description;
    mongoc_topology_scanner_t *scanner;
@@ -63,13 +65,14 @@ mongoc_topology_reconcile (mongoc_topology_t *topology) {
    scanner = topology->scanner;
 
    /* Add newly discovered nodes */
-   mongoc_set_for_each(description->servers,
-                       _mongoc_topology_reconcile_add_nodes,
-                       topology);
+   mongoc_set_for_each (
+      description->servers, _mongoc_topology_reconcile_add_nodes, topology);
 
    /* Remove removed nodes */
-   DL_FOREACH_SAFE (scanner->nodes, ele, tmp) {
-      if (!mongoc_topology_description_server_by_id (description, ele->id, NULL)) {
+   DL_FOREACH_SAFE (scanner->nodes, ele, tmp)
+   {
+      if (!mongoc_topology_description_server_by_id (
+             description, ele->id, NULL)) {
          mongoc_topology_scanner_node_retire (ele);
       }
    }
@@ -78,24 +81,23 @@ mongoc_topology_reconcile (mongoc_topology_t *topology) {
 
 /* call this while already holding the lock */
 static bool
-_mongoc_topology_update_no_lock (uint32_t            id,
-                                 const bson_t       *ismaster_response,
-                                 int64_t             rtt_msec,
-                                 mongoc_topology_t  *topology,
+_mongoc_topology_update_no_lock (uint32_t id,
+                                 const bson_t *ismaster_response,
+                                 int64_t rtt_msec,
+                                 mongoc_topology_t *topology,
                                  const bson_error_t *error /* IN */)
 {
-   mongoc_topology_description_handle_ismaster (&topology->description, id,
-                                                ismaster_response, rtt_msec,
-                                                error);
+   mongoc_topology_description_handle_ismaster (
+      &topology->description, id, ismaster_response, rtt_msec, error);
 
    /* The processing of the ismaster results above may have added/removed
     * server descriptions. We need to reconcile that with our monitoring agents
     */
-   mongoc_topology_reconcile(topology);
+   mongoc_topology_reconcile (topology);
 
    /* return false if server removed from topology */
-   return mongoc_topology_description_server_by_id (&topology->description,
-                                                    id, NULL) != NULL;
+   return mongoc_topology_description_server_by_id (
+             &topology->description, id, NULL) != NULL;
 }
 
 
@@ -143,23 +145,41 @@ _mongoc_topology_scanner_setup_err_cb (uint32_t id,
  */
 
 void
-_mongoc_topology_scanner_cb (uint32_t            id,
-                             const bson_t       *ismaster_response,
-                             int64_t             rtt_msec,
-                             void               *data,
+_mongoc_topology_scanner_cb (uint32_t id,
+                             const bson_t *ismaster_response,
+                             int64_t rtt_msec,
+                             void *data,
                              const bson_error_t *error /* IN */)
 {
    mongoc_topology_t *topology;
+   mongoc_server_description_t *sd;
 
    BSON_ASSERT (data);
 
-   topology = (mongoc_topology_t *)data;
+   topology = (mongoc_topology_t *) data;
 
    mongoc_mutex_lock (&topology->mutex);
-   _mongoc_topology_update_no_lock (id, ismaster_response, rtt_msec, topology,
-                                    error);
+   sd = mongoc_topology_description_server_by_id (
+      &topology->description, id, NULL);
+
+   /* Server Discovery and Monitoring Spec: "Once a server is connected, the
+    * client MUST change its type to Unknown only after it has retried the
+    * server once." */
+   if (!ismaster_response && sd && sd->type != MONGOC_SERVER_UNKNOWN) {
+      _mongoc_topology_update_no_lock (
+         id, ismaster_response, rtt_msec, topology, error);
+
+      /* add another ismaster call to the current scan - the scan continues
+       * until all commands are done */
+      mongoc_topology_scanner_scan (
+         topology->scanner, sd->id, topology->connect_timeout_msec);
+   } else {
+      _mongoc_topology_update_no_lock (
+         id, ismaster_response, rtt_msec, topology, error);
+
+      mongoc_cond_broadcast (&topology->cond_client);
+   }
 
-   mongoc_cond_broadcast (&topology->cond_client);
    mongoc_mutex_unlock (&topology->mutex);
 }
 
@@ -179,8 +199,7 @@ _mongoc_topology_scanner_cb (uint32_t            id,
  *-------------------------------------------------------------------------
  */
 mongoc_topology_t *
-mongoc_topology_new (const mongoc_uri_t *uri,
-                     bool                single_threaded)
+mongoc_topology_new (const mongoc_uri_t *uri, bool single_threaded)
 {
    int64_t heartbeat_default;
    int64_t heartbeat;
@@ -191,7 +210,7 @@ mongoc_topology_new (const mongoc_uri_t *uri,
 
    BSON_ASSERT (uri);
 
-   topology = (mongoc_topology_t *)bson_malloc0(sizeof *topology);
+   topology = (mongoc_topology_t *) bson_malloc0 (sizeof *topology);
 
    /*
     * Not ideal, but there's no great way to do this.
@@ -200,10 +219,10 @@ mongoc_topology_new (const mongoc_uri_t *uri,
     *   - otherwise, if the seed list has a single host, initialize to SINGLE
     *   - everything else gets initialized to UNKNOWN
     */
-   if (mongoc_uri_get_replica_set(uri)) {
+   if (mongoc_uri_get_replica_set (uri)) {
       init_type = MONGOC_TOPOLOGY_RS_NO_PRIMARY;
    } else {
-      hl = mongoc_uri_get_hosts(uri);
+      hl = mongoc_uri_get_hosts (uri);
       if (hl->next) {
          init_type = MONGOC_TOPOLOGY_UNKNOWN;
       } else {
@@ -211,24 +230,26 @@ mongoc_topology_new (const mongoc_uri_t *uri,
       }
    }
 
-   heartbeat_default = single_threaded ?
-                       MONGOC_TOPOLOGY_HEARTBEAT_FREQUENCY_MS_SINGLE_THREADED :
-                       MONGOC_TOPOLOGY_HEARTBEAT_FREQUENCY_MS_MULTI_THREADED;
+   heartbeat_default =
+      single_threaded ? MONGOC_TOPOLOGY_HEARTBEAT_FREQUENCY_MS_SINGLE_THREADED
+                      : MONGOC_TOPOLOGY_HEARTBEAT_FREQUENCY_MS_MULTI_THREADED;
 
-   heartbeat = mongoc_uri_get_option_as_int32 (uri, "heartbeatfrequencyms",
-                                               heartbeat_default);
+   heartbeat = mongoc_uri_get_option_as_int32 (
+      uri, MONGOC_URI_HEARTBEATFREQUENCYMS, heartbeat_default);
 
-   mongoc_topology_description_init (&topology->description, init_type,
-                                     heartbeat);
+   mongoc_topology_description_init (
+      &topology->description, init_type, heartbeat);
 
-   topology->description.set_name = bson_strdup(mongoc_uri_get_replica_set(uri));
+   topology->description.set_name =
+      bson_strdup (mongoc_uri_get_replica_set (uri));
 
    topology->uri = mongoc_uri_copy (uri);
    topology->scanner_state = MONGOC_TOPOLOGY_SCANNER_OFF;
-   topology->scanner = mongoc_topology_scanner_new (topology->uri,
-                                                    _mongoc_topology_scanner_setup_err_cb,
-                                                    _mongoc_topology_scanner_cb,
-                                                    topology);
+   topology->scanner =
+      mongoc_topology_scanner_new (topology->uri,
+                                   _mongoc_topology_scanner_setup_err_cb,
+                                   _mongoc_topology_scanner_cb,
+                                   topology);
 
    topology->single_threaded = single_threaded;
    if (single_threaded) {
@@ -242,20 +263,18 @@ mongoc_topology_new (const mongoc_uri_t *uri,
        *   "The serverSelectionTryOnce option MUST be true by default."
        */
       topology->server_selection_try_once = mongoc_uri_get_option_as_bool (
-         uri,
-         "serverselectiontryonce",
-         true);
+         uri, MONGOC_URI_SERVERSELECTIONTRYONCE, true);
    } else {
       topology->server_selection_try_once = false;
    }
 
-   topology->server_selection_timeout_msec = mongoc_uri_get_option_as_int32(
-      topology->uri, "serverselectiontimeoutms",
+   topology->server_selection_timeout_msec = mongoc_uri_get_option_as_int32 (
+      topology->uri,
+      MONGOC_URI_SERVERSELECTIONTIMEOUTMS,
       MONGOC_TOPOLOGY_SERVER_SELECTION_TIMEOUT_MS);
 
-   topology->local_threshold_msec = mongoc_uri_get_option_as_int32 (
-      topology->uri, "localthresholdms",
-      MONGOC_TOPOLOGY_LOCAL_THRESHOLD_MS);
+   topology->local_threshold_msec =
+      mongoc_uri_get_local_threshold_option (topology->uri);
 
    /* Total time allowed to check a server is connectTimeoutMS.
     * Server Discovery And Monitoring Spec:
@@ -264,18 +283,18 @@ mongoc_topology_new (const mongoc_uri_t *uri,
     *   regular sockets. Multi-threaded clients SHOULD set monitoring sockets'
     *   socketTimeoutMS to the connectTimeoutMS."
     */
-   topology->connect_timeout_msec = mongoc_uri_get_option_as_int32(
-      topology->uri, "connecttimeoutms",
-      MONGOC_DEFAULT_CONNECTTIMEOUTMS);
+   topology->connect_timeout_msec =
+      mongoc_uri_get_option_as_int32 (topology->uri,
+                                      MONGOC_URI_CONNECTTIMEOUTMS,
+                                      MONGOC_DEFAULT_CONNECTTIMEOUTMS);
 
    mongoc_mutex_init (&topology->mutex);
    mongoc_cond_init (&topology->cond_client);
    mongoc_cond_init (&topology->cond_server);
 
-   for ( hl = mongoc_uri_get_hosts (uri); hl; hl = hl->next) {
-      mongoc_topology_description_add_server (&topology->description,
-                                              hl->host_and_port,
-                                              &id);
+   for (hl = mongoc_uri_get_hosts (uri); hl; hl = hl->next) {
+      mongoc_topology_description_add_server (
+         &topology->description, hl->host_and_port, &id);
       mongoc_topology_scanner_add (topology->scanner, hl, id);
    }
 
@@ -291,14 +310,16 @@ mongoc_topology_new (const mongoc_uri_t *uri,
  *-------------------------------------------------------------------------
  */
 void
-mongoc_topology_set_apm_callbacks (mongoc_topology_t      *topology,
+mongoc_topology_set_apm_callbacks (mongoc_topology_t *topology,
                                    mongoc_apm_callbacks_t *callbacks,
-                                   void                   *context)
+                                   void *context)
 {
    if (callbacks) {
-      memcpy (&topology->description.apm_callbacks, callbacks,
+      memcpy (&topology->description.apm_callbacks,
+              callbacks,
               sizeof (mongoc_apm_callbacks_t));
-      memcpy (&topology->scanner->apm_callbacks, callbacks,
+      memcpy (&topology->scanner->apm_callbacks,
+              callbacks,
               sizeof (mongoc_apm_callbacks_t));
    }
 
@@ -332,13 +353,13 @@ mongoc_topology_destroy (mongoc_topology_t *topology)
    _mongoc_topology_description_monitor_closed (&topology->description);
 
    mongoc_uri_destroy (topology->uri);
-   mongoc_topology_description_destroy(&topology->description);
+   mongoc_topology_description_destroy (&topology->description);
    mongoc_topology_scanner_destroy (topology->scanner);
    mongoc_cond_destroy (&topology->cond_client);
    mongoc_cond_destroy (&topology->cond_server);
    mongoc_mutex_destroy (&topology->mutex);
 
-   bson_free(topology);
+   bson_free (topology);
 }
 
 
@@ -354,7 +375,7 @@ mongoc_topology_destroy (mongoc_topology_t *topology)
  */
 static void
 _mongoc_topology_do_blocking_scan (mongoc_topology_t *topology,
-                                   bson_error_t      *error)
+                                   bson_error_t *error)
 {
    mongoc_topology_scanner_t *scanner;
 
@@ -363,12 +384,10 @@ _mongoc_topology_do_blocking_scan (mongoc_topology_t *topology,
    _mongoc_handshake_freeze ();
 
    scanner = topology->scanner;
-   mongoc_topology_scanner_start (scanner,
-                                  (int32_t) topology->connect_timeout_msec,
-                                  true);
+   mongoc_topology_scanner_start (
+      scanner, (int32_t) topology->connect_timeout_msec, true);
 
-   mongoc_topology_scanner_work (topology->scanner,
-                                 topology->connect_timeout_msec);
+   mongoc_topology_scanner_work (topology->scanner);
 
    mongoc_mutex_lock (&topology->mutex);
    _mongoc_topology_scanner_finish (scanner);
@@ -384,26 +403,32 @@ _mongoc_topology_do_blocking_scan (mongoc_topology_t *topology,
 
 bool
 mongoc_topology_compatible (const mongoc_topology_description_t *td,
-                            const mongoc_read_prefs_t           *read_prefs,
-                            bson_error_t                        *error)
+                            const mongoc_read_prefs_t *read_prefs,
+                            bson_error_t *error)
 {
    int64_t max_staleness_seconds;
    int32_t max_wire_version;
 
+   if (td->compatibility_error.code) {
+      memcpy (error, &td->compatibility_error, sizeof (bson_error_t));
+      return false;
+   }
+
    if (!read_prefs) {
       /* NULL means read preference Primary */
       return true;
    }
 
-   max_staleness_seconds = mongoc_read_prefs_get_max_staleness_seconds (
-      read_prefs);
+   max_staleness_seconds =
+      mongoc_read_prefs_get_max_staleness_seconds (read_prefs);
 
    if (max_staleness_seconds != MONGOC_NO_MAX_STALENESS) {
-      max_wire_version = mongoc_topology_description_lowest_max_wire_version (
-         td);
+      max_wire_version =
+         mongoc_topology_description_lowest_max_wire_version (td);
 
       if (max_wire_version < WIRE_VERSION_MAX_STALENESS) {
-         bson_set_error (error, MONGOC_ERROR_COMMAND,
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
                          MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
                          "Not all servers support maxStalenessSeconds");
          return false;
@@ -411,14 +436,15 @@ mongoc_topology_compatible (const mongoc_topology_description_t *td,
 
       /* shouldn't happen if we've properly enforced wire version */
       if (!mongoc_topology_description_all_sds_have_write_date (td)) {
-         bson_set_error (error, MONGOC_ERROR_COMMAND,
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
                          MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
                          "Not all servers have lastWriteDate");
          return false;
       }
 
       if (!_mongoc_topology_description_validate_max_staleness (
-         td, max_staleness_seconds, error)) {
+             td, max_staleness_seconds, error)) {
          return false;
       }
    }
@@ -428,21 +454,23 @@ mongoc_topology_compatible (const mongoc_topology_description_t *td,
 
 
 static void
-_mongoc_server_selection_error (const char         *msg,
+_mongoc_server_selection_error (const char *msg,
                                 const bson_error_t *scanner_error,
-                                bson_error_t       *error)
+                                bson_error_t *error)
 {
    if (scanner_error && scanner_error->code) {
       bson_set_error (error,
                       MONGOC_ERROR_SERVER_SELECTION,
                       MONGOC_ERROR_SERVER_SELECTION_FAILURE,
                       "%s: %s",
-                      msg, scanner_error->message);
+                      msg,
+                      scanner_error->message);
    } else {
       bson_set_error (error,
                       MONGOC_ERROR_SERVER_SELECTION,
                       MONGOC_ERROR_SERVER_SELECTION_FAILURE,
-                      "%s", msg);
+                      "%s",
+                      msg);
    }
 }
 
@@ -475,13 +503,13 @@ _mongoc_server_selection_error (const char         *msg,
  *-------------------------------------------------------------------------
  */
 mongoc_server_description_t *
-mongoc_topology_select (mongoc_topology_t         *topology,
-                        mongoc_ss_optype_t         optype,
+mongoc_topology_select (mongoc_topology_t *topology,
+                        mongoc_ss_optype_t optype,
                         const mongoc_read_prefs_t *read_prefs,
-                        bson_error_t              *error)
+                        bson_error_t *error)
 {
-   uint32_t server_id = mongoc_topology_select_server_id (topology, optype,
-                                                          read_prefs, error);
+   uint32_t server_id =
+      mongoc_topology_select_server_id (topology, optype, read_prefs, error);
 
    if (server_id) {
       /* new copy of the server description */
@@ -504,10 +532,10 @@ mongoc_topology_select (mongoc_topology_t         *topology,
  *-------------------------------------------------------------------------
  */
 uint32_t
-mongoc_topology_select_server_id (mongoc_topology_t         *topology,
-                                  mongoc_ss_optype_t         optype,
+mongoc_topology_select_server_id (mongoc_topology_t *topology,
+                                  mongoc_ss_optype_t optype,
                                   const mongoc_read_prefs_t *read_prefs,
-                                  bson_error_t              *error)
+                                  bson_error_t *error)
 {
    static const char *timeout_msg =
       "No suitable servers found: `serverSelectionTimeoutMS` expired";
@@ -517,7 +545,7 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
    bool try_once;
    int64_t sleep_usec;
    bool tried_once;
-   bson_error_t scanner_error = { 0 };
+   bson_error_t scanner_error = {0};
    int64_t heartbeat_msec;
    uint32_t server_id;
 
@@ -534,8 +562,8 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
    local_threshold_ms = topology->local_threshold_msec;
    try_once = topology->server_selection_try_once;
    loop_start = loop_end = bson_get_monotonic_time ();
-   expire_at = loop_start
-               + ((int64_t) topology->server_selection_timeout_msec * 1000);
+   expire_at =
+      loop_start + ((int64_t) topology->server_selection_timeout_msec * 1000);
 
    if (topology->single_threaded) {
       _mongoc_topology_description_monitor_opening (&topology->description);
@@ -551,15 +579,16 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
       for (;;) {
          if (topology->stale) {
             /* how soon are we allowed to scan? */
-            scan_ready = topology->last_scan
-                         + MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS * 1000;
+            scan_ready = topology->last_scan +
+                         MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS * 1000;
 
             if (scan_ready > expire_at && !try_once) {
                /* selection timeout will expire before min heartbeat passes */
                _mongoc_server_selection_error (
                   "No suitable servers found: "
                   "`serverselectiontimeoutms` timed out",
-                  &scanner_error, error);
+                  &scanner_error,
+                  error);
 
                topology->stale = true;
                return 0;
@@ -576,16 +605,13 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
             tried_once = true;
          }
 
-         if (!mongoc_topology_compatible (&topology->description, read_prefs,
-                                          error)) {
+         if (!mongoc_topology_compatible (
+                &topology->description, read_prefs, error)) {
             return 0;
          }
 
          selected_server = mongoc_topology_description_select (
-            &topology->description,
-            optype,
-            read_prefs,
-            local_threshold_ms);
+            &topology->description, optype, read_prefs, local_threshold_ms);
 
          if (selected_server) {
             return selected_server->id;
@@ -597,7 +623,8 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
             if (tried_once) {
                _mongoc_server_selection_error (
                   "No suitable servers found (`serverSelectionTryOnce` set)",
-                  &scanner_error, error);
+                  &scanner_error,
+                  error);
 
                return 0;
             }
@@ -606,8 +633,8 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
 
             if (loop_end > expire_at) {
                /* no time left in server_selection_timeout_msec */
-               _mongoc_server_selection_error (timeout_msg,
-                                               &scanner_error, error);
+               _mongoc_server_selection_error (
+                  timeout_msg, &scanner_error, error);
 
                return 0;
             }
@@ -620,22 +647,20 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
    for (;;) {
       mongoc_mutex_lock (&topology->mutex);
 
-      if (!mongoc_topology_compatible (&topology->description, read_prefs,
-                                       error)) {
+      if (!mongoc_topology_compatible (
+             &topology->description, read_prefs, error)) {
          mongoc_mutex_unlock (&topology->mutex);
          return 0;
       }
 
       selected_server = mongoc_topology_description_select (
-         &topology->description,
-         optype,
-         read_prefs,
-         local_threshold_ms);
+         &topology->description, optype, read_prefs, local_threshold_ms);
 
-      if (! selected_server) {
+      if (!selected_server) {
          _mongoc_topology_request_scan (topology);
 
-         r = mongoc_cond_timedwait (&topology->cond_client, &topology->mutex,
+         r = mongoc_cond_timedwait (&topology->cond_client,
+                                    &topology->mutex,
                                     (expire_at - loop_start) / 1000);
 
          mongoc_topology_scanner_get_error (topology->scanner, &scanner_error);
@@ -647,24 +672,23 @@ mongoc_topology_select_server_id (mongoc_topology_t         *topology,
          if (r == ETIMEDOUT) {
 #endif
             /* handle timeouts */
-            _mongoc_server_selection_error (timeout_msg,
-                                            &scanner_error, error);
+            _mongoc_server_selection_error (timeout_msg, &scanner_error, error);
 
             return 0;
          } else if (r) {
-            bson_set_error(error,
-                           MONGOC_ERROR_SERVER_SELECTION,
-                           MONGOC_ERROR_SERVER_SELECTION_FAILURE,
-                           "Unknown error '%d' received while waiting on "
-                           "thread condition", r);
+            bson_set_error (error,
+                            MONGOC_ERROR_SERVER_SELECTION,
+                            MONGOC_ERROR_SERVER_SELECTION_FAILURE,
+                            "Unknown error '%d' received while waiting on "
+                            "thread condition",
+                            r);
             return 0;
          }
 
          loop_start = bson_get_monotonic_time ();
 
          if (loop_start > expire_at) {
-            _mongoc_server_selection_error (timeout_msg,
-                                            &scanner_error, error);
+            _mongoc_server_selection_error (timeout_msg, &scanner_error, error);
 
             return 0;
          }
@@ -709,9 +733,8 @@ mongoc_topology_server_by_id (mongoc_topology_t *topology,
    mongoc_mutex_lock (&topology->mutex);
 
    sd = mongoc_server_description_new_copy (
-      mongoc_topology_description_server_by_id (&topology->description,
-                                                id,
-                                                error));
+      mongoc_topology_description_server_by_id (
+         &topology->description, id, error));
 
    mongoc_mutex_unlock (&topology->mutex);
 
@@ -752,9 +775,8 @@ _mongoc_topology_host_by_id (mongoc_topology_t *topology,
    mongoc_mutex_lock (&topology->mutex);
 
    /* not a copy - direct pointer into topology description data */
-   sd = mongoc_topology_description_server_by_id (&topology->description,
-                                                  id,
-                                                  error);
+   sd = mongoc_topology_description_server_by_id (
+      &topology->description, id, error);
 
    if (sd) {
       host = bson_malloc0 (sizeof (mongoc_host_list_t));
@@ -797,15 +819,15 @@ _mongoc_topology_request_scan (mongoc_topology_t *topology)
  *--------------------------------------------------------------------------
  */
 void
-mongoc_topology_invalidate_server (mongoc_topology_t  *topology,
-                                   uint32_t            id,
+mongoc_topology_invalidate_server (mongoc_topology_t *topology,
+                                   uint32_t id,
                                    const bson_error_t *error)
 {
    BSON_ASSERT (error);
 
    mongoc_mutex_lock (&topology->mutex);
-   mongoc_topology_description_invalidate_server (&topology->description,
-                                                  id, error);
+   mongoc_topology_description_invalidate_server (
+      &topology->description, id, error);
    mongoc_mutex_unlock (&topology->mutex);
 }
 
@@ -826,7 +848,7 @@ mongoc_topology_invalidate_server (mongoc_topology_t  *topology,
  *--------------------------------------------------------------------------
  */
 bool
-_mongoc_topology_update_from_handshake (mongoc_topology_t                 *topology,
+_mongoc_topology_update_from_handshake (mongoc_topology_t *topology,
                                         const mongoc_server_description_t *sd)
 {
    bool has_server;
@@ -836,9 +858,15 @@ _mongoc_topology_update_from_handshake (mongoc_topology_t                 *topol
 
    mongoc_mutex_lock (&topology->mutex);
 
-   has_server = _mongoc_topology_update_no_lock (sd->id, &sd->last_is_master,
-                                                 sd->round_trip_time_msec,
-                                                 topology, NULL);
+   mongoc_topology_description_handle_ismaster (&topology->description,
+                                                sd->id,
+                                                &sd->last_is_master,
+                                                sd->round_trip_time_msec,
+                                                NULL);
+
+   /* return false if server was removed from topology */
+   has_server = mongoc_topology_description_server_by_id (
+                   &topology->description, sd->id, NULL) != NULL;
 
    /* if pooled, wake threads waiting in mongoc_topology_server_by_id */
    mongoc_cond_broadcast (&topology->cond_client);
@@ -863,8 +891,7 @@ _mongoc_topology_update_from_handshake (mongoc_topology_t                 *topol
  *--------------------------------------------------------------------------
  */
 int64_t
-mongoc_topology_server_timestamp (mongoc_topology_t *topology,
-                                  uint32_t           id)
+mongoc_topology_server_timestamp (mongoc_topology_t *topology, uint32_t id)
 {
    mongoc_topology_scanner_node_t *node;
    int64_t timestamp = -1;
@@ -920,8 +947,8 @@ _mongoc_topology_get_type (mongoc_topology_t *topology)
  *
  *--------------------------------------------------------------------------
  */
-static
-void * _mongoc_topology_run_background (void *data)
+static void *
+_mongoc_topology_run_background (void *data)
 {
    mongoc_topology_t *topology;
    int64_t now;
@@ -934,7 +961,7 @@ void * _mongoc_topology_run_background (void *data)
    BSON_ASSERT (data);
 
    last_scan = 0;
-   topology = (mongoc_topology_t *)data;
+   topology = (mongoc_topology_t *) data;
    heartbeat_msec = topology->description.heartbeat_msec;
 
    /* we exit this loop when shutdown_requested, or on error */
@@ -944,7 +971,8 @@ void * _mongoc_topology_run_background (void *data)
 
       /* we exit this loop on error, or when we should scan immediately */
       for (;;) {
-         if (topology->shutdown_requested) goto DONE;
+         if (topology->shutdown_requested)
+            goto DONE;
 
          now = bson_get_monotonic_time ();
 
@@ -956,18 +984,19 @@ void * _mongoc_topology_run_background (void *data)
 
          timeout = heartbeat_msec - ((now - last_scan) / 1000);
 
-         /* if someone's specifically asked for a scan, use a shorter interval */
+         /* if someone's specifically asked for a scan, use a shorter interval
+          */
          if (topology->scan_requested) {
-            force_timeout = MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS - ((now - last_scan) / 1000);
+            force_timeout = MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS -
+                            ((now - last_scan) / 1000);
 
             timeout = BSON_MIN (timeout, force_timeout);
          }
 
          /* if we can start scanning, do so immediately */
          if (timeout <= 0) {
-            mongoc_topology_scanner_start (topology->scanner,
-                                           topology->connect_timeout_msec,
-                                           false);
+            mongoc_topology_scanner_start (
+               topology->scanner, topology->connect_timeout_msec, false);
             break;
          } else {
             /* otherwise wait until someone:
@@ -975,12 +1004,13 @@ void * _mongoc_topology_run_background (void *data)
              *   o we time out
              *   o requests a shutdown
              */
-            r = mongoc_cond_timedwait (&topology->cond_server, &topology->mutex, timeout);
+            r = mongoc_cond_timedwait (
+               &topology->cond_server, &topology->mutex, timeout);
 
 #ifdef _WIN32
-            if (! (r == 0 || r == WSAETIMEDOUT)) {
+            if (!(r == 0 || r == WSAETIMEDOUT)) {
 #else
-            if (! (r == 0 || r == ETIMEDOUT)) {
+            if (!(r == 0 || r == ETIMEDOUT)) {
 #endif
                /* handle errors */
                goto DONE;
@@ -995,8 +1025,7 @@ void * _mongoc_topology_run_background (void *data)
 
       /* scanning locks and unlocks the mutex itself until the scan is done */
       mongoc_mutex_unlock (&topology->mutex);
-      mongoc_topology_scanner_work (topology->scanner,
-                                    topology->connect_timeout_msec);
+      mongoc_topology_scanner_work (topology->scanner);
 
       mongoc_mutex_lock (&topology->mutex);
 
@@ -1007,7 +1036,7 @@ void * _mongoc_topology_run_background (void *data)
       topology->last_scan = bson_get_monotonic_time ();
       mongoc_mutex_unlock (&topology->mutex);
 
-      last_scan = bson_get_monotonic_time();
+      last_scan = bson_get_monotonic_time ();
    }
 
 DONE:
@@ -1048,8 +1077,8 @@ _mongoc_topology_start_background_scanner (mongoc_topology_t *topology)
       _mongoc_handshake_freeze ();
       _mongoc_topology_description_monitor_opening (&topology->description);
 
-      r = mongoc_thread_create (&topology->thread,
-                                _mongoc_topology_run_background, topology);
+      r = mongoc_thread_create (
+         &topology->thread, _mongoc_topology_run_background, topology);
 
       if (r != 0) {
          MONGOC_ERROR ("could not start topology scanner thread: %s",
@@ -1092,7 +1121,8 @@ _mongoc_topology_background_thread_stop (mongoc_topology_t *topology)
       mongoc_cond_signal (&topology->cond_server);
       topology->scanner_state = MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN;
       join_thread = true;
-   } else if (topology->scanner_state == MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN) {
+   } else if (topology->scanner_state ==
+              MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN) {
       /* if we're mid shutdown, wait until it shuts down */
       while (topology->scanner_state != MONGOC_TOPOLOGY_SCANNER_OFF) {
          mongoc_cond_wait (&topology->cond_client, &topology->mutex);
@@ -1112,15 +1142,13 @@ _mongoc_topology_background_thread_stop (mongoc_topology_t *topology)
 }
 
 bool
-_mongoc_topology_set_appname (mongoc_topology_t *topology,
-                              const char *appname)
+_mongoc_topology_set_appname (mongoc_topology_t *topology, const char *appname)
 {
    bool ret = false;
    mongoc_mutex_lock (&topology->mutex);
 
    if (topology->scanner_state == MONGOC_TOPOLOGY_SCANNER_OFF) {
-      ret = _mongoc_topology_scanner_set_appname (topology->scanner,
-                                                  appname);
+      ret = _mongoc_topology_scanner_set_appname (topology->scanner, appname);
    } else {
       MONGOC_ERROR ("Cannot set appname after handshake initiated");
    }
